<?php

include_once "CQuartet.php";

class CQSet {

  var $MidiGen = array();
  var $pause_gap = 15; // pause for sound between quartets
  var $pause_sib = 120; // pause for sibelius between quartets
  var $pause_finish = 180;
  var $scount = 0; // Number of set elements. 0 for automatic count sconst_type
  var $sconst_type = array(0, 2, 3); // Either maximum type or 0 for range
  var $sconst_type_min = 0;
  var $sconst_type_max = 3;
  var $vcount = 6; // Number of voices
  var $ncount_base = 70; // Basic ncount
  var $ncount_rand = 30; // Additional random ncount
  var $mode_follow_p = 100; // Probability to follow current mode when making new Quartet
  var $steps_skip_p = array(0 => 70, 1 => 30); // Probabilities of skipping first steps
  var $Q = array(); // Array of CQuartet classes
  
  function __construct() {
    if ($this->scount == 0) $this->scount = count($this->sconst_type);
  }
  
  protected function SetConst($s, $type) {
    // Sets constants for CQuartet
    $Q = $this->Q[$s];
    echo "Q$s set type to $type<br>";
    if ($type == 1) {
	    // Fast dissonating chromatic
	    $Q->len_ch_p[ 60] = array(120 =>   0, 60 =>   3, 45 =>   0, 30 =>  80, 24 =>   5, 20 =>   5, 16 =>  0, 15 =>   0);
	    $Q->len_ch_p[ 45] = array(120 =>   0, 60 =>   3, 45 =>   0, 30 =>  10, 24 =>   0, 20 =>   0, 16 =>  0, 15 =>   0);
	    $Q->len_ch_p[ 30] = array(120 =>   0, 60 =>  10, 45 =>   0, 30 =>   5, 24 =>  50, 20 =>  40, 16 =>  0, 15 =>   0);
	    $Q->len_ch_p[ 24] = array(120 =>   0, 60 =>   3, 45 =>   0, 30 =>  20, 24 =>  20, 20 =>  40, 16 =>  0, 15 =>   0);
	    $Q->len_ch_p[ 20] = array(120 =>   0, 60 =>   3, 45 =>   0, 30 =>  20, 24 =>  10, 20 => 100, 16 =>  0, 15 => 400);
	    $Q->len_ch_p[ 16] = array(120 =>   0, 60 =>   3, 45 =>   0, 30 =>  10, 24 =>   0, 20 =>   0, 16 =>  0, 15 =>   0);
	    $Q->len_ch_p[ 15] = array(120 =>   0, 60 =>   0, 45 =>   0, 30 =>  10, 24 =>  10, 20 => 200, 16 =>  0, 15 => 400);
	    //$Q->voice_repeat_mul = array (0.00001, 0.00001, 0.00001);
	    $Q->voice_repeat_r = -1;
	    $Q->try_fine = 10;
	    $Q->dia_p = 0;
	    $Q->try2_fine = 10;
	    $Q->note_alter_fine_max = 10;
	    $Q->diss_max = 100;
	    $Q->si_max = 100;
	    $Q->resolution_max = 100;
	    $Q->contra_max = 100;
	    $Q->cht_dir_p = array(100, 100);
	    $Q->chi_dir_p = array(100, 100);
	    $Q->chs_dir_p = array(100, 100);
	    $Q->mode_div = 120;
	    $Q->mode_mut_p = 90;
	    $Q->mode_mut_min = -3;
	    $Q->mode_mut_max = 3;
      $Q->CreateNotes(0, 36, array(12, 6, 3, 3, 3));
    }
    if ($type == 2) {
      // Fast non-arpegiating, no diatonic
      //$Q->ncount = round($Q->ncount * 1.2);
	    $Q->len_ch_p[ 60] = array(120 =>   0, 60 =>   3, 45 =>   0, 30 =>  80, 24 =>   5, 20 =>   5, 16 =>  0, 15 =>   0);
	    $Q->len_ch_p[ 45] = array(120 =>   0, 60 =>   3, 45 =>   0, 30 =>  10, 24 =>   0, 20 =>   0, 16 =>  0, 15 =>   0);
	    $Q->len_ch_p[ 30] = array(120 =>   0, 60 =>  10, 45 =>   0, 30 =>   5, 24 =>  50, 20 =>  40, 16 =>  0, 15 =>   0);
	    $Q->len_ch_p[ 24] = array(120 =>   0, 60 =>   3, 45 =>   0, 30 =>  20, 24 =>  20, 20 =>  40, 16 =>  0, 15 =>   0);
	    $Q->len_ch_p[ 20] = array(120 =>   0, 60 =>   3, 45 =>   0, 30 =>  20, 24 =>  10, 20 => 100, 16 =>  0, 15 => 400);
	    $Q->len_ch_p[ 16] = array(120 =>   0, 60 =>   3, 45 =>   0, 30 =>  10, 24 =>   0, 20 =>   0, 16 =>  0, 15 =>   0);
	    $Q->len_ch_p[ 15] = array(120 =>   0, 60 =>   0, 45 =>   0, 30 =>  10, 24 =>  10, 20 => 200, 16 =>  0, 15 => 400);
	    $Q->voice_repeat_r = -1;
	    $Q->dia_p = 0;
    }
    if ($type == 3) {
      // Slow
      $Q->ncount = round($Q->ncount * 0.5);
	    $Q->len_ch_p[ 60] = array(120 =>   0, 60 =>  40, 45 =>   8, 30 =>  80, 24 =>   5, 20 =>   5, 16 =>  8, 15 =>   0);
	    $Q->len_ch_p[ 45] = array(120 =>   0, 60 =>  20, 45 =>  80, 30 =>  10, 24 =>   0, 20 =>   0, 16 =>  0, 15 =>   0);
	    $Q->len_ch_p[ 30] = array(120 =>   0, 60 =>  20, 45 =>   8, 30 =>  10, 24 =>  50, 20 =>  40, 16 =>  8, 15 =>   0);
	    $Q->len_ch_p[ 24] = array(120 =>   0, 60 =>  20, 45 =>   0, 30 =>  20, 24 =>   8, 20 =>   0, 16 =>  0, 15 =>   0);
	    $Q->len_ch_p[ 20] = array(120 =>   0, 60 =>  20, 45 =>   0, 30 =>  20, 24 =>  10, 20 =>  10, 16 =>  0, 15 =>  10);
	    $Q->len_ch_p[ 16] = array(120 =>   0, 60 =>  20, 45 =>   0, 30 =>  10, 24 =>   0, 20 =>   0, 16 => 80, 15 =>   0);
	    $Q->len_ch_p[ 15] = array(120 =>   0, 60 =>   0, 45 =>   0, 30 =>  10, 24 =>  10, 20 =>  20, 16 =>  0, 15 =>  40);
    }
  }
  
  function Generate() {
    $mode_key = 0;
    for ($s=0; $s<$this->scount; $s++) {
      // Create ncount
      $ncount = $this->ncount_base + mt_rand(0, $this->ncount_rand);
      // New object
      $Q = new CQuartet($this->vcount);
      $this->Q[$s] = $Q;
      // Follow mode
      if (($s>0) && (rnd(0, 99) <= $this->mode_follow_p)) {
        //echo "Q$s mode set to $mode_key<br>";
        $Q->SetMode(0, $mode_key);
        $Q->CreateNotes(0, 36+$mode_key, array(12, 4, 3, 5, 4));
      }
      // Set constants
      if (is_array($this->sconst_type)) {
        $type = $this->sconst_type[$s];
      } else {
        $type = mt_rand($this->sconst_type_min, $this->sconst_type_max);
      }
      $Q->ncount = $ncount;
      //echo "Q$s ncount set to $ncount<br>";
      $this->SetConst($s, $type);
      // Generate
      $Q->Generate();
      // Prolong last step if there is no culmination
      if ($Q->culm[$Q->ncount-1] == 0) $Q->pause[$Q->ncount-1] = $this->pause_finish;
      // Save mode
      $mode_key = $Q->mode_key[$Q->ncount-1];
      // Save quartet
    }
  }
  
  function Show() {
    echo "\r\n<table><tr>";
    for ($s=0; $s<$this->scount; $s++) {
      echo "<td>";
      $this->Q[$s]->Show($s>0);
      echo "</td>";
    }
    echo "</tr></table>";
  }

  function ShowStat() {
    echo "<table><tr>";
    for ($s=0; $s<$this->scount; $s++) {
      echo "<td>";
      $this->Q[$s]->ShowStat($s>0);
      echo "</td>";
    }
    echo "</tr></table>";
  }

  function ShowLegend() {
    $this->Q[0]->ShowLegend();
    /*
    $Q = $this->Q[0];
    foreach ($Q->ch_a as $key => $val) {
      echo "C$key, ";
    }
    */
  }

  function GenerateMidi() {
		for ($sib=0; $sib<=1; $sib++) {
			$this->MidiGen[$sib] = new CMidiGen;
			$posv=$this->MidiGen[$sib]->PositionV;
			$pos = $posv;
			for ($s=0; $s<$this->scount; $s++) {
  		  // Set skip
			  $skip = 0;
			  if ($s>0) {
			    $skip = rand_pa($this->steps_skip_p);
			  }
			  for ($v=0; $v<$this->Q[$s]->vcount; $v++) {
				  // Export
				  $this->MidiGen[$sib]->PositionV = $posv;
				  $this->MidiGen[$sib]->AddVoice($this->Q[$s], $v, $sib, $skip);
				  $pos = max($this->MidiGen[$sib]->Position, $pos);
				  //echo "Q$s voice $v ended at pos $pos<br>";
				}
				$posv = $pos;
				if ($sib == 0) {
				  $posv += $this->MidiGen[$sib]->Step * $this->pause_gap;
				} else {
				  $left = $this->Q[$s]->measure - (($this->Q[$s]->time[$this->Q[$s]->ncount-1] + $this->Q[$s]->len[$this->Q[$s]->ncount-1]) % $this->Q[$s]->measure);
				  //echo "Q$s Detected left $left<br>";
				  $posv += $this->MidiGen[$sib]->Step * ($left + $this->pause_sib);
				}
			}
			$this->MidiGen[$sib]->StartTrack();
			$this->MidiGen[$sib]->Compile(0);
			$this->MidiGen[$sib]->EndTrack();
			$this->MidiGen[$sib]->StartTrack("Piano");
			$this->MidiGen[$sib]->Compile(1);
			$this->MidiGen[$sib]->EndTrack();
		}

		// Save txt to files
		$fh=fopen("temp-midi.txt", "w");
		fwrite($fh, $this->MidiGen[0]->txt);
		fclose($fh);
  
		$fh=fopen("temp-midi-sib.txt", "w");
		fwrite($fh, $this->MidiGen[1]->txt);
		fclose($fh);
  
		// Save midi to files
		$midi = new Midi();
		$midi->importTxt($this->MidiGen[0]->txt);
		$midi->saveMidFile("temp.mid", 0666);
		unset($midi);
		
		$midi = new Midi();
		$midi->importTxt($this->MidiGen[1]->txt);
		$midi->saveMidFile("temp-sib.mid", 0666);
		unset($midi);
  }
}

?>