<?php

include "genlib.php";
include "CQShow.php";

class CQuartet extends CQShow {
  
  protected function GetNoteLim($i, $v, &$lim1, &$lim2) {
    // Count limits
    switch ($v) {
      case 0:
        $lim1 = $this->note_min[$i];
        $lim2 = $this->note[$v + 1][$i];
        break;
      case ($this->vcount - 1):
        $lim1 = $this->note[$v - 1][$i];
        $lim2 = $this->note_max[$i];
        break;
      default:
        $lim1 = $this->note[$v - 1][$i];
        $lim2 = $this->note[$v + 1][$i];
    }
  }
  
  protected function GetNoteWin($i, $v, &$win1, &$win2, $p=0) {
    // If $p is 1, then probabilities are included for real limits calculations
    // If $p is 0, then general limits are calculated
    $lim1 = 0;
    $lim2 = 0;
    $this->GetNoteLim($i, $v, $lim1, $lim2);
    // Move limits
    $prime = 0;
    if ($p) if (rnd(0, 99) < $this->prime_p) $prime = 1;
    if ($prime == 0) {
      $lim1 = mode_walk($this->mode[$i], $lim1, 1);
      $lim2 = mode_walk($this->mode[$i], $lim2, -1);
    }
    // Check if limits moved over note
    if ($lim1>$this->note[$v][$i]) $lim1 = $this->note[$v][$i];
    if ($lim2<$this->note[$v][$i]) $lim2 = $this->note[$v][$i];
    // Choose deviations
    $d0=$this->note_d_def[0];
    $d1=$this->note_d_def[1];
    foreach ($this->note_d as $key => $val) {
      if ($this->note[$v][$i] < $key) {
        $d0 = $val[0];
        $d1 = $val[1];
      }
    }
    // Create window
    $win1 = $this->note[$v][$i] - $d0;
    $win2 = $this->note[$v][$i] + $d1;
    if ($win1 < $lim1)
      $win1 = $lim1;
    if ($win2 > $lim2) $win2 = $lim2;
    // Cut window near the limits
    if ($p) {
      // Count distances to upper and lower limits
      $dist1=$this->note[$v][$i] - $this->note_min[$i];
      $dist2=$this->note_max[$i] - $this->note[$v][$i];
      // Check if notes are out of range
      if ($dist1<0) $dist1=0;
      if ($dist2<0) $dist2=0;
      // Count probability of cutting window based on the distance to the note_min and note_max
      if ($dist1<=$this->note_grav_dist1) {
        $prob1=$this->note_grav_p1;
      } else {
        $prob1=$this->note_grav_p1 - $this->note_grav_decr1 * ($dist1 - $this->note_grav_dist1);
      }
      if ($dist2<=$this->note_grav_dist2) {
        $prob2=$this->note_grav_p2;
      } else {
        $prob2=$this->note_grav_p2 - $this->note_grav_decr2 * ($dist2 - $this->note_grav_dist2);
      }
      // Cut limits
      $this->note_g[$v][$i]=0;
      if ((rnd(0, 99) < $prob1) && ($this->note_free2[$v][$i-1] >= $this->note_grav_minfree1)) {
        $win1=$this->note[$v][$i];
        $this->note_g[$v][$i]=1;
        //echo "$v:$i $win1:$win2<br>";
      }
      if ((rnd(0, 99) < $prob2) && ($this->note_free1[$v][$i-1] >= $this->note_grav_minfree2)) {
        $win2=$this->note[$v][$i];
        $this->note_g[$v][$i]=-1;
        //echo "$v:$i $win1:$win2<br>";
      }
    }
    if ($win1>$win2) echo "<font color=red>Error $win1>$win2 for $v:$i</font><br>";
  }
  
  protected function NoteMut_Move($i, $v, $mcount) {
    //echo "/v$v/i$i/$lim1/$lim2/$win1/$win2/</font><br>";
    $this->note_same[$v][$i]=0;
    $same=0;
    $win1=0;
    $win2=0;
    $this->GetNoteWin($i, $v, $win1, $win2, 1);
    // Random move
    $passed = 0;
    $this->retry_voice[$i]=0;
    for($try = 0; $try < $this->try_max; $try ++) {
      $mv = mt_rand($win1 - 1, $win2 - 1);
      // echo "n$mv<br>";
      $mv = mode_walk($this->mode[$i], $mv, 1);
      // Check interval
      $int=abs($mv-$this->note[$v][$i]);
      $passed=1;
      $same=0;
      if (in_array($int, $this->bad_mv)) $passed=0;
      if ($int==0) {
        if (rnd(0, 99)>=$this->samenote_p[$this->mcount[$i]]) {
          $passed = 0;
        } else {
          //echo "$v:$i ".$this->note[$v][$i]." Same<br>";
          $same=1;
        }
      }
      if ($passed==1) break;
      // Not passed
      $this->retry_voice[$i]++;
    }
    $this->note[$v][$i] = $mv;
    $this->note_same[$v][$i] = $same;
    // echo "n$mv<br>";
  }

  protected function AnalyseDiss($i, $final=0) {
    $dn1=array();
    $dn2=array();
    $this->diss_str[$i]="";
    $dcount = 0;
    // Clear results
    for ($v=0; $v<$this->vcount; $v++) {
      unset($this->note_diss_a[$i][$v]);
      $this->note_diss_a[$i][$v] = array();
      $this->note_diss[$v][$i]=0;
      $this->dvcount[$v][$i]=0;
      $this->dvcount2[$v][$i]=0;
      $this->note_dw[$v][$i]=0;
    }
    // Find dissonance intervals:
    // dvcount[v][i] - total number of dissonance weights for note
    // dvcount2[v][i] - Number of diss with particular voice (without regard to note fade out)
    // note_dw[v][i] - maximum dissonance weight for note
    // note_diss[v][i] - the shortest dissonating interval that note participates in
    // note_res[v][i] - if not zero, shows that note should resolve up (1) or down (-1) diatonically
    // note_diss_a[i][v][v2] - if not zero, shows that note dissonates with v2 note
    // Clear resolution indication
    for ($v = 0; $v < $this->vcount; $v ++) $this->note_res[$v][$i] = 0;
    for ($v = 0; $v < $this->vcount - 1; $v ++) {
      for($v2 = $v + 1; $v2 < $this->vcount; $v2 ++) {
        $note1=$this->note[$v][$i];
        $note2=$this->note[$v2][$i];
        $int = $note2 - $note1;
        // Check bass intervals
        $da = $this->diss_a_def;
        $da2 = array();
        foreach ($this->diss_a as $key => $val) {
          if (($note1 < $key) && ($note2 < $key)) $da2 = $val;
        }
        // Check if interval is dissonating
        $wei = max($da[$int % 12], $da2[$int]);
        if ($wei > 0) {
          // Set variables before 
          $this->dvcount2[$v][$i] += $wei;
          $this->dvcount2[$v2][$i] += $wei;
          // This is weight reduction due to note left volume
          if (($this->note[$v][$i-1] == $note1) || ($this->note[$v2][$i-1] == $note2)) {
            $wei *= $this->diss_pedal/100;
            $max = 129;
            $c = 0.25;
            $xx = $this->vol_left[$v][$i-1]/$max;
            if ($this->note[$v][$i-1] == $note1) $wei *= (1 - pow(1 - $xx, $this->diss_w_vol))*(1-$c) + pow($xx, 1/$this->diss_w_vol)*$c;
            $xx = $this->vol_left[$v2][$i-1]/$max;
            if ($this->note[$v2][$i-1] == $note2) $wei *= (1 - pow(1 - $xx, $this->diss_w_vol))*(1-$c) + pow($xx, 1/$this->diss_w_vol)*$c;
          }
          // Set local variables
          $this->note_diss_a[$i][$v][$v2] = $wei;
          $this->note_diss_a[$i][$v2][$v] = $wei;
          $this->dvcount[$v][$i] += $wei;
          $this->dvcount[$v2][$i] += $wei;
          if ($this->note_dw[$v][$i] < $wei) $this->note_dw[$v][$i]=$wei;
          if ($this->note_dw[$v2][$i] < $wei) $this->note_dw[$v2][$i]=$wei;
          // Find shortest intervals for both notes
          if (($this->note_diss[$v][$i]==0) || (abs($this->note_diss[$v][$i])>abs($int))) {
            $this->note_diss[$v][$i]=$int;
            $this->note_dv[$v][$i]=$v2;
          }
          if (($this->note_diss[$v2][$i]==0) || (abs($this->note_diss[$v2][$i])>abs($int))) { 
            $this->note_diss[$v2][$i]=-$int;
            $this->note_dv[$v2][$i]=$v;
          }
          // Check resolution rules if both notes were hit
          if (($this->note[$v][$i] != $this->note[$v][$i-1]) || ($this->retrigger[$v][$i])) 
          if (($this->note[$v2][$i] != $this->note[$v2][$i-1]) || ($this->retrigger[$v2][$i])) {
            if ($this->note_res[$v][$i] == 0)  $this->note_res[$v][$i] =  $this->resolution1[$int]; 
            if ($this->note_res[$v2][$i] == 0) $this->note_res[$v2][$i] = $this->resolution2[$int % 12];
            //if (($final) && ($this->resolution2[$int % 12] != 0)) echo "<font color=red>$i:$v detected resolution with $v2 of int $int</font><br>"; 
          }
          $dcount += $wei;
          $copy=0;
          //$this->log("- $i:$v:$v2 Int $int ".($da[$int % 12])." da2 ".($da2[$int])." wei ".$wei);
          // Find copy of this dissonance ignoring octaves
          for ($x=0; $x<count($dn1); $x++) {
            if (($dn1[$x]==($note1 % 12)) && ($dn2[$x]==($note2 %12))) {
              $copy=1;
              break;
            }
            if (($dn2[$x]==($note1 %12)) && ($dn1[$x]==($note2 %12))) {
              $copy=1;
              break;
            }
          }
          if ($copy==0) {
            $dn1[]=$note1 % 12;
            $dn2[]=$note2 % 12;
          }
        }
      }
      //if (($final) && ($this->note_res[$v][$i] != 0)) echo "$i:$v resolution ".$this->note_res[$v][$i]."<br>";
    }
    //if (($final) && ($this->note_res[$this->vcount-1][$i] != 0)) echo "$i:".($this->vcount-1)." resolution ".$this->note_res[$this->vcount-1][$i]."<br>";
    for ($x=0; $x<count($dn1); $x++) $this->diss_str[$i].=get_notename($dn1[$x]%12).":".get_notename($dn2[$x]%12)." ";
    $dcount *= $this->diss_c;
    $this->dcount[$i] = $dcount;
    //echo "$i:$v Dcount $dcount<br>";
    return $dcount;
  }
  
  protected function AnalyseIM($i) {
    // Analyse interval movements
    $icount = 0;
    for ($v=0; $v<$this->vcount; $v++) {
      $this->note_si[$v][$i]=0;
    }
    for ($v = 0; $v < $this->vcount - 1; $v ++) {
      for ($v2 = $v + 1; $v2 < $this->vcount; $v2 ++) {
        $note1=$this->note[$v][$i];
        $note2=$this->note[$v2][$i];
        $int = $note2 - $note1;
        $note3 = $this->note[$v][$i-1];
        $note4 = $this->note[$v2][$i-1];
        $int2 = $note4 - $note3;
        if (in_array($int % 12, $this->int_strict)) {
          // Parallel and antiparallel
          if ((($int % 12) == ($int2 % 12)) && (($note1-$note3)*($note2-$note4) != 0)) {
            $wei=1;
            $icount += $wei;
            $this->note_si[$v][$i] += $wei;
            $this->note_si[$v2][$i] += $wei;
          } else
          // Hidden intervals
          if (($note1-$note3)*($note2-$note4)>0) {
            $wei=0.5;
            $icount += $wei;
            $this->note_si[$v][$i] += $wei;
            $this->note_si[$v2][$i] += $wei;
          }
        }
      }
    }
    $this->icount[$i]=$icount;
    return $icount;
  }
  
  protected function FindResolutions($i, $final=0) {
    // This function returns number of resolution violations
    $rcount=0;
    for ($v=0; $v<$this->vcount; $v++) {
      $this->note_rv[$v][$i]=0;
    }
    for ($v = 0; $v < $this->vcount; $v ++) {
      $int = $this->note[$v][$i] - $this->note[$v][$i-1];
      // Check if note is moving and note was dissonating
      if (($this->mv[$i][$v] > 0) && ($this->dvcount[$v][$i-1] > 0)) {
        // One type of resolution violation is when none of two voices moves diatonically down
        if (($int>-1) || ($int<-2)) {
          if ($v<$this->vcount-1) for ($v2 = $v + 1; $v2 < $this->vcount; $v2 ++) {
            if (($this->mv[$i][$v2] > 0) && ($this->note_diss_a[$i-1][$v][$v2] != 0)) {
              $int2 = $this->note[$v2][$i] - $this->note[$v2][$i-1];
              if (($int2>-1) || ($int2<-2)) {
                $rcount += 1;
                $this->note_rv[$v][$i] = 1;
                $this->note_rv[$v2][$i] = 1;
                /*
                if ($final) {
                  echo "$i:$v:$v2 Both moving wrong ";
                  print_r($this->mv[$i]);
                  echo "<br>";
                }
                */
              }
            }
          }
        }
        // Second type of resolution violation is strict dissonating voice moving in wrong direction or not diatonically
        // Pay attention that all resolutions must be down or else this code should be modified **** 
        if ($this->note_res[$v][$i-1]<0) {
          if (($int>-1) || ($int<-2)) {
            $rcount += 1;
            $this->note_rv[$v][$i] = 1;
            //if ($final) echo "$i:$v:$v2 Strict resolution violation<br>";
          }
        }
      }
    }
    $this->rcount[$i]=$rcount;
    return $rcount;
  }
  
  protected function NoteMut_Check($i, $state=0) {
    // State = 0 for normal check
    // State = 1 for note_alter check
    $fine=0;
    // Count diss number
    $dcount = $this->AnalyseDiss($i);
    if ($dcount > $this->diss_max) $fine+=$dcount - $this->diss_max;
    // Count resolution violations
    $rcount = $this->FindResolutions($i);
    if ($rcount > $this->resolution_max) $fine+=$rcount - $this->resolution_max;
    // Count strict int number
    $icount = $this->AnalyseIM($i);
    if ($icount > $this->si_max) $fine+=$icount - $this->si_max;
    // Count contradictions number
    $ccount = $this->FindContra($i);
    if ($ccount > $this->contra_max) $fine+=$ccount - $this->contra_max;
    // Count loiter number
    $lcount = $this->FindLoiter($i);
    $lmax = $this->loiter_max;
    if ($state == 1) $lmax = $this->loiter_max_alter;
    if ($lcount > $lmax) $fine+=$lcount - $lmax;
    // Find violation of previous locks in case of 100% lock - this is mostly for reprises, because it cannot happen in normal notemut
    $unlocks=0;
    for($v = 0; $v < $this->vcount; $v ++) {
      if (($this->note_locked[$v][$i-1]>0) && ($this->note[$v][$i] != $this->note[$v][$i-1])) {
        $lock_c = $this->note[$v][$i-1];
        // Add fine only in case of 100% lock
        if ($lock_c == 0) {
	        $unlocks++;
        	echo "<font color=red>$i:$v Found lock violation ".$this->note[$v][$i-1]." to ".$this->note[$v][$i]." over ".$this->note_locked[$v][$i-1]." state $state</font><br>";
        }
      }
    }
    $fine += $unlocks * $this->unlock_fine;
    // Count orchestral density
    $scount = $this->AnalyseDensity($i);
    if (rnd(0, 99) >= $this->dens_allow_p) $fine += abs($scount) * $this->dens_fine; 
    //if ($fine == 0 ) echo "$i:$v Fine 0<br>";
    // Log
    //$st = "$i F$fine D".round($dcount, 2)." C".round($ccount, 2)." I".round($icount, 2)." R".round($rcount, 2)." ";
    //for ($v=0; $v<$this->vcount; $v++) if ($this->mv[$i][$v]>0) $st.=($this->note[$v][$i])." ".get_notename($this->note[$v][$i])." ";
    //$this->log($st);
    return $fine;
  }

  protected function CopyNote($i1, $i2, $v1, $v2, $save=0) {
    if ($save == 3) {
      $cond = $this->note_changed[$i1][$v1];
      // Check if there was no movement, then count retriggers too
      if ($this->rmcount[$i1] == 0) $cond = $this->hit[$i1][$v1];
      if ($cond) {
        // Check if note already exists in other voice, then retrigger it and do not copy
        $exist = 0;
        for ($v3=0; $v3<$this->vcount; $v3++) {
          // Skip my voice
          if ($v3 == $v1) continue;
          // Compare target note with other voice in $i2
          if ($this->note[$v3][$i2] == $this->note[$v1][$i1]) {
            echo "$i2:$v3 Detected move to existing voice<br>";
            $exist = 1;
            break;
          }
        }
        if ($exist) {
	        $this->retrigger[$v3][$i2] = 1;
        } else {
	        // Copy note 
	        $this->note[$v2][$i2] = $this->note[$v1][$i1];
	        // Check if moved note is the same and retrigger it
	        if ($this->note[$v2][$i2] == $this->note[$v2][$i2-1]) {
	          $this->retrigger[$v2][$i2] = 1;
	        }
        }
      }
    } else {
      $this->note[$v2][$i2] = $this->note[$v1][$i1];
    }
    if ($save) {
      $this->note_g[$v2][$i2] = $this->note_g[$v1][$i1];
      $this->note_si[$v2][$i2] = $this->note_si[$v1][$i1];
      $this->note_same[$v2][$i2] = $this->note_same[$v1][$i1];
      // Copy this for retriggermut
      $this->dvcount2[$v2][$i2] = $this->dvcount2[$v1][$i1];
      $this->dvcount[$v2][$i2] = $this->dvcount[$v1][$i1];
    }
    // Do not need these - they are analysed and overwritten
    //$this->dvcount[$v][$i2] = $this->dvcount[$v][$i1];
    //$this->vol[$v][$i2] = $this->vol[$v][$i1];
  }
  
  protected function CopyPos($i1, $i2, $save=0) {
    // Save equals 1 when saving values
    // Save equals 2 when saving values for NoteMut
    // Save equals 3 when copying notes for reprise
    //$this->mode_key[$i2] = $this->mode_key[$i1];
    //$this->mode[$i2] = $this->mode[$i1];
    if ($save) {
      $this->fine[$i2] = $this->fine[$i1];
      $this->rcount[$i2] = $this->rcount[$i1];
      $this->dcount[$i2] = $this->dcount[$i1];
      $this->icount[$i2] = $this->icount[$i1];
      $this->ccount[$i2] = $this->ccount[$i1];
      $this->diss_str[$i2] = $this->diss_str[$i1];
    }
    if ($save>1) {
      $this->mcount[$i2] = $this->mcount[$i1];
      $this->mv[$i2] = $this->mv[$i1];
      $this->mvp[$i2] = $this->mvp[$i1];
    }
    for($v = 0; $v < $this->vcount; $v ++) {
      $this->CopyNote($i1, $i2, $v, $v, $save);
    }
  }
  
  protected function NormalizeVoices($i) {
    $bpos=$this->ncount+1002;
    $na = array();
    // Create array
    for($v = 0; $v < $this->vcount; $v ++) {
      $na[$v] = $this->note[$v][$i];
    }
    // Sort array
    asort($na);
    // Return notes back to note array
    $v=0;
    foreach ($na as $key => $val) {
      // Check if not was normalized
      if ($val != $this->note[$v][$i]) {
        echo "Detected crossed notes $v:$key at $i<br>";
        for($v2 = 0; $v2 < $this->vcount; $v2 ++) {
          if ($this->note[$v2][$i] == $val) {
            // Check if notes moved
            if ($this->note[$v][$i] == $this->note[$v][$i-1]) $this->note_normalized[$i][$v2]=1;
            if ($this->note[$v2][$i] == $this->note[$v2][$i-1]) $this->note_normalized[$i][$v]=1;
            echo "Crossed notes $v:$v2 resolved at $i and normalized ".$this->note_normalized[$i][$v].":".$this->note_normalized[$i][$v2]."<br>";
            $this->CopyNote($i, $bpos, $v, $v);
            $this->CopyNote($i, $i, $v2, $v);
            $this->CopyNote($bpos, $i, $v, $v2);
          }
        }
      }
      $v++;
    }
  }

  protected function CopyVol($i1, $i2) {
    for($v = -1; $v <= $this->vcount; $v ++) {
      $this->vol[$v][$i2] = $this->vol[$v][$i1];
      $this->vol_sm[$v][$i2] = $this->vol_sm[$v][$i1];
      $this->vol_vel[$v][$i2] = $this->vol_vel[$v][$i1];
      $this->vol_acc[$v][$i2] = $this->vol_acc[$v][$i1];
    }
  }

  protected function GetLock_c($pitch) {
    $lock_c = 0;
    foreach ($this->locked_c as $key => $val) {
      if ($pitch < $key) {
        $lock_c=$val;
        break;
      }
    }
    return $lock_c;
  }
  
  protected function NoteMut2($i, $bad_mv) {
	  // Position for saving best NoteMut_Move results
    $bpos=$this->ncount+1000;
    // Choose moving voices number
    $mcount_p=$this->mcount_p;
    $time1=$this->time[$i] % $this->measure;
    foreach ($mcount_p as $key => $val) {
      $mcount_p[$key] *= $this->len_mcount_mul[$this->len[$i]][$key];
      if (isset($this->time_mcount_mul[$time1])) $mcount_p[$key] *= $this->time_mcount_mul[$time1][$key];
    }
    //echo $this->time[$i].":$time1 "; print_r($mcount_p); echo "<br>";
    $mcount = rand_pa($mcount_p);
    if ($mcount==0) {
      print_r($mcount_p);
      echo " <font color=red>$i Mcount $mcount </font><br>";
      $mcount=1; 
    }
    // $mcount = mt_rand($this->mcount_min, $this->mcount_max);
    $mv_p=$this->mv_p;
    // For voices above vcount set probability to 0 
    for ($v=$this->vcount; $v<10; $v++) $mv_p[$v]=0;
    // Increase probability for bass to move after culmination
    if ($this->culm[$i-1]) {
      if (rnd(0, 99) < $this->culm_bass_p) {
        $mv_p[0] *= $this->culm_bass_mul;
        $mv_p[1] *= $this->culm_bass_mul/3;
        //echo "Bass increased at $i<br>";
      }
    }
    // Change mv_p considering lock, smother and diss
    for ($v=0; $v<$this->vcount; $v++) {
      $free=max($this->note_free1[$v][$i-1], $this->note_free2[$v][$i-1]);
      $smo = $this->smo_a_def;
      foreach ($this->smo_a as $key => $val) {
        if ($this->note[$v][$i-1] < $key) $smo = $val;
      }
      // Increase probability to move down for culminations
      // Increase probability for discords
      if ($this->note_diss[$v][$i-1]!=0) {
        $this->mvp_diss[$i][$v] = pow($this->diss_mul_p, $this->dvcount[$v][$i-1]);
        $mv_p[$v] *= $this->mvp_diss[$i][$v];
      }
      // Decrease probability for smothered
      if ($free < $smo) {
        $this->mvp_smo[$i][$v] = $this->smo_mul_p;
        $mv_p[$v] *= $this->mvp_smo[$i][$v];
      }
      // Decrease probability for locked
      $lock_c = $this->GetLock_c($this->note[$v][$i-1]);
      if ($this->note_locked[$v][$i-1] > 0) $mv_p[$v] *= $lock_c;
			// Increase probability for too long notes
			//echo "NL: ".$this->note_len[$v][$i-1]."<br>";
			if ($this->note_len[$v][$i-1] > $this->note_max_len[$v]) {
			  $this->mvp_len[$i][$v] = $this->note_max_len_mul[$v] * $this->note_len[$v][$i-1]/$this->note_max_len[$v]; 
			  $mv_p[$v] *= $this->mvp_len[$i][$v];
			  //echo "NML: ".($this->note_max_len_mul[$v] * $this->note_len[$v][$i-1]/$this->note_max_len[$v])."<br>";
			}
			// Increase probability for near notes
			if (($i>1) && ($this->note[$v][$i-1] != $this->note[$v][$i-2]) && (rnd(0, 99)<$this->voice_repeat_p)) {
			  $pos=0;
			  if ($this->voice_repeat_r>-1) for ($x=-$this->voice_repeat_r; $x<=$this->voice_repeat_r; $x++) {
			    if (($v+$x >= 0) && ($v+$x < $this->vcount)) {
			      $this->mvp_near[$i][$v+$x] = $this->voice_repeat_mul[$pos] * $this->voice_repeat_len_mul[$this->len[$i]] * $this->voice_repeat_mcount_mul[$mcount]; 
			      $mv_p[$v+$x] *= $this->mvp_near[$i][$v+$x];
			    }
			    $pos++;
			  }
			}
			// Decrease probability for notes that lead to high fine in previous try
			if ($bad_mv[$v]>0) $mv_p[$v] /= $this->try2_mv * $bad_mv[$v];
    }
    // Record mv_p for display
    $this->mvp[$i] = $mv_p;
    //echo "$i "; print_r($mv_p); echo "<br>";
    // Choose moving voices
    $this->mv[$i] = array();
    for($x = 0; $x < $mcount; $x ++) {
      $mvn = rand_pa($mv_p);
      // Record voice number
      $this->mv[$i][$mvn] = 1;
      // Protect from choosing this voice again
      $mv_p[$mvn]=0;
    }
    // Correct mcount
    $mcount = array_sum($this->mv[$i]);
    $this->mcount[$i] = $mcount;
    ksort($this->mv[$i], SORT_NUMERIC);
    //echo "$mcount ";
    //print_r($this->mv[$i]); echo "<br>";
    // Choose moves direction
    $this->fine[$bpos]=100000;
    $passed = 0;
    //$this->retry[$i]=0;
    for($try = 0; $try < $this->try_max * pow($this->try_mul, $mcount-1); $try ++) {
      $this->CopyPos($i - 1, $i);
      for($v = 0; $v < $this->vcount; $v ++) {
        if (one($this->mv[$i][$v]))
          $this->NoteMut_Move($i, $v, $mcount);
      }
      // Check rules
      $fine=$this->NoteMut_Check($i);
      $this->fine[$i]=$fine;
      if ($fine <= $this->try_fine) {
        $passed = 1;
        break;
      } else {
        $this->retry[$i]++;
        if ($this->fine[$bpos]>$fine) {
          //echo "$i:$try:$fine:".$this->fine[$bpos]."<br>";
          $this->CopyPos($i, $bpos, 1);
        }
			  $this->sfine[$try] += $this->fine[$bpos];
      }
    }
    if ($passed==0) $this->CopyPos($bpos, $i, 1);
    // echo "MC$mcount<br>";
    $this->mcount[$i] = $mcount;
    return $this->fine[$i];
  }

  protected function DiaAnalyse($i, $dia, $dir) {
    // Analyse future diatonic movement
    $fine = 0;
    for ($x=0; $x<$this->dia_steps_analyse; $x++) {
      $p1 = $i-1+$x;
      $p2 = $i+$x;
      $this->CopyPos($p1, $p2);
      // Remove locks because dia pitch is high enough
      for ($v=0; $v<$this->vcount; $v++) {
        $this->note_locked[$v][$p2] = 0;
      }
      if ($p2 > $i) {
        $this->mode[$p2] = $this->mode[$p1];
        $this->note_max[$p2] = $this->note_max[$p1];
        $this->note_min[$p2] = $this->note_min[$p1];
        $this->len[$p2] = $this->len[$p1];
      }
      $this->mv[$p2] = array();
      $this->mv[$p2][$dia] = 1;
      $this->mcount[$p2] = 1;
      $this->note[$dia][$p2] = mode_walk($this->mode[$p2], $this->note[$dia][$p1], $dir);
      //if (($x == 0) && (abs($this->note[$dia][$p2] - $this->note[$dia][$p1]) == 1)) echo "$i:$p1:$p2:$dia:$dir DiaA semitone ".$this->note[$dia][$p2]."<br>";
      $fine1 = $this->NoteMut_Check($p2);
      $fine += $fine1;
      //$this->log("$i:$p1:$p2:$dia:$dir DiaA $fine1 ".$this->note[$dia][$p2]);
      $this->NoteAnalyse($p2);
    }
    return $fine / $this->dia_steps_analyse;
  }
  
  protected function NoteMut_Dia($i) {
    // Check if movement is too long
    if ($this->dia_count[$i-1] >= $this->dia_max) return 0;
    // Check if end of piece
    if ($i > $this->ncount - $this->dia_stop) return 0;
    // Check for random cut
    if (rnd(0, 99) < $this->dia_cut_p) return 0;
    // Check if diatonic movement is in process
    if ($this->dia[$i-1]>0) {
      $dia = $this->dia[$i-1];
      $dir = $this->dia_dir[$i-1]/abs($this->dia_dir[$i-1]);
      // Check if len is not too long
      if ($this->len[$i] > $this->dia_len_max2) return 0;
      // Check for masked tritone
      $note = mode_walk($this->mode[$i], $this->note[$dia][$i-1], $dir);
      for ($x=1; $x<30; $x++) {
        $p1 = $i-$x;
        if ($p1<0) break;
        if ($this->note[$dia][$p1] == $this->note[$dia][$p1+1]) break;
        if (abs($this->note[$dia][$p1] - $note) == 6) {
          //echo "$i Masked tritone at $p1 between $note and ".$this->note[$dia][$p1]."<br>";
          $dir *= 2;
          break;
        }
      }
      $note = mode_walk($this->mode[$i], $this->note[$dia][$i-1], $dir);
      // Check if there is enough space for moving
      $free = ($this->note[$dia+$dir/abs($dir)][$i-1] - $note)*$dir/abs($dir);
      if ($free < 1) return 0;
    } else {
      // Check if we can start diatonic movement
      // Check if len is not too long
      if ($this->len[$i] > $this->dia_len_max) return 0;
      //echo "$i Len passed<br>";
      // Check if space before is enough
      $x2=0;
      for ($x=$i; $x>0; $x--) {
        if ($this->dia[$x]>0) return 0;
        $x2++;
        if ($x2 >= $this->dia_space) break;
      }
      //echo "$i Space passed<br>";
      // Check if we have enough interval between voices, and high enough notes
      for ($v=$this->dia_v_min; $v<$this->vcount-1; $v++) {
        if ($this->note[$v+1][$i-1] - $this->note[$v][$i-1] > $this->dia_int_min)
          if ($this->note[$v][$i-1] > $this->dia_pitch_min) $dia=$v;
      }
      if ($dia == 0) return 0;
      //echo "$i Voice $dia passed<br>";
      // Check probability of diatonic movement
      if (rnd(0, 99) >= $this->dia_p) return 0;
      // Choose direction of movement
      $dir = 1;
      //$this->log("dia $dia 1");
      $fine1 = $this->DiaAnalyse($i, $dia, 1);
      //$this->log("dia $dia -1");
      $fine2 = $this->DiaAnalyse($i, $dia+1, -1);
      // Check if there is no low fine
      if (min($fine1, $fine2) > $this->dia_fine_max) return 0;
      //echo "$i Dia fine ".round($fine1, 2)." ".round($fine2, 2)."<br>";
      if ($fine1 > $fine2) {
        $dir = -1;
        $dia++;
      }  
      if ($fine1 == $fine2) {
        if (rnd(0, 99) < 50) {
          $dir = -1;
          $dia++;
        }
      }
      //echo "<b>$i Accepted dia $dia $dir</b><br>";
    }
    // Make diatonic movement
    $this->CopyPos($i - 1, $i);
    $this->note[$dia][$i] = mode_walk($this->mode[$i], $this->note[$dia][$i-1], $dir);
    $this->mv[$i] = array();
    $this->mv[$i][$dia] = 1;
    $this->mcount[$i] = 1;
    $fine = $this->NoteMut_Check($i);
    $this->fine[$i] = $fine;
    // Check if fine is too big
    if ($fine > $this->dia_fine_max2) {
      //echo "$i Finished ";
      //if ($this->dia[$i-1] == 0) echo "starting"; 
      //echo " dia $dia:$dir due to fine $fine ($fine1:$fine2) with note ".$this->note[$dia][$i]."<br>";
      return 0;
    }
    //if ($this->dia[$i-1] == 0) echo "$i <b>Dia started</b> $dir fine ".round($fine1, 2)." ".round($fine2, 2)." ".round($fine, 2)."<br>";
    $this->dia[$i] = $dia;
    $this->dia_dir[$i] = $dir;
    $this->dia_count[$i] = $this->dia_count[$i-1] + 1;
    return $dia;
  } 
  
  protected function NoteMut($i) {
    // Check if diatonic movement is in process or we can start it
    if ($this->NoteMut_Dia($i) > 0) return 0;
	  // Position for saving best NoteMut results
    $bpos=$this->ncount+1001;
    $this->fine[$bpos]=100000;
    $bad_mv=array();
    for($try = 0; $try < $this->try2_max; $try ++) {
      $fine = $this->NoteMut2($i, $bad_mv);
      if ($fine < $this->try2_fine) {
        $passed = 1;
        break;
      } else {
        for ($v=0; $v<$this->vcount; $v++) $bad_mv[$v] += $this->mv[$i][$v];
        //echo " $i Try2 $try Fine $fine "; print_r($this->mv[$i]); echo "<br>";
        $this->retry2[$i]++;
        if ($this->fine[$bpos]>$fine) {
          //echo "$i:$try:$fine:".$this->fine[$bpos]."<br>";
          $this->CopyPos($i, $bpos, 2);
        }
      }
    }
    if ($passed==0) $this->CopyPos($bpos, $i, 2);
    // echo "MC$mcount<br>";
    return $this->fine[$i];
  }
  
  protected function FindContra($i, $same_voice=0) {
    // Same_voice shows if we have to look for contradictions in the same voice
    // Find contradicting notes
    $ccount=0;
		$this->ccount[$i]=0;
    $mode=$this->mode[$i];
    if ($this->mode[$i-1] != $mode) $prev_mode=$this->mode[$i-1];
    else if (($i>1) && ($this->mode[$i-2] != $mode)) $prev_mode=$this->mode[$i-2];
		else $prev_mode = $mode;
    for ($v=0; $v<$this->vcount; $v++) {
		  $this->note_cont[$v][$i] = 0;
      $note = $this->note[$v][$i];
      // Check if note is not in the mode and it is hit
      $hit=0;
      if (($this->note[$v][$i] != $this->note[$v][$i-1]) || ($this->retrigger[$v][$i])) $hit=1;
      if (($prev_mode[$note % 12] == 0) && ($hit)) {
        for ($v2=0; $v2<$this->vcount; $v2++) {
          if (($v2==$v) && ($same_voice == 0)) continue;
          $int1=abs($note - $this->note[$v2][$i-1]) % 12;
          $int2=abs($note - $this->note[$v2][$i-2]) % 12;
          $hit=0;
          if (($this->note[$v2][$i-1] != $this->note[$v2][$i-2]) || ($this->retrigger[$v2][$i-1])) $hit=1;
          if ((($int1 == 1) || ($int1 == 11) || ($int1 == 6)) && ($hit)) {
            $ccount += 1;
            $this->note_cont[$v][$i] += 1;
          }
          $hit=0;
          if (($this->note[$v2][$i-2] != $this->note[$v2][$i-3]) || ($this->retrigger[$v2][$i-2])) $hit=1;
          if ((($int2 == 1) || ($int2 == 11)) && ($hit)) {
            $ccount += 1;
            $this->note_cont[$v][$i] += 1;
          }
        }
      }
    }
    $this->ccount[$i]=$ccount;
    //if ($ccount>0) echo "Contra $i<br>";
    return $ccount;
  }
  
  protected function FindLoiter($i) {
    // Loitering notes are notes that were close (or tritone-away) to our current notes, which (current notes) are new and appeared with the newly born mode
    // Find loitering notes
    $lcount=0;
		$this->lcount[$i]=0;
    $mode=$this->mode[$i];
    // Determine previous mode either mode of the $i-1 step or $i-2 step (if $i-1 mode is the same as $i)
    if ($this->mode[$i-1] != $mode) $prev_mode=$this->mode[$i-1];
    else if (($i>1) && ($this->mode[$i-2] != $mode)) $prev_mode=$this->mode[$i-2];
		else $prev_mode = $mode;
    for ($v=0; $v<$this->vcount; $v++) {
		  $this->note_loi[$v][$i] = 0;
      $note = $this->note[$v][$i];
      //echo "$i:$v Loiter? <br>";
      // Check if note is hit
      $hit=0;
      if (($this->note[$v][$i] != $this->note[$v][$i-1]) || ($this->retrigger[$v][$i])) $hit=1;
      // Check if current note is not in the previous mode and it is hit
      if (($prev_mode[$note % 12] == 0) && ($hit)) {
        //echo "$i:$v Checking loiter <br>";
        // For this note check intervals between it and all the notes of the previous x steps 
        for ($v2=0; $v2<$this->vcount; $v2++) {
          //if (($v2==$v) && ($same_voice == 0)) continue;
          for ($x=1; $x<=$this->loiter_steps; $x++) {
            $pos = $i - $x;
            $int=abs($note - $this->note[$v2][$pos]) % 12;
            if ($int == 11) $int=1;
            if ($int == 6) $int=1;
            // Check if this note is hit
            $hit=0;
            if (($this->note[$v2][$pos] != $this->note[$v2][$pos-1]) || ($this->retrigger[$v2][$pos])) $hit=1;
            //echo "$i:$v Checking loiter int $int with $v2:$pos, lcount $lcount, hit $hit<br>";
            // Check if interval is dissonating
            if (($int <= $this->loiter_int) && ($hit)) {
              $lcount += 1;
              $this->note_loi[$v][$i] += 1;
            }
          }
        }
      }
    }
    $this->lcount[$i]=$lcount;
    //if ($lcount>0) echo "$i Loiter $lcount<br>";
    return $lcount;
  }
  
  protected function GetChord($i) {
    $this->chord[$i]="";
    $this->chord_i[$i]="";
    $this->chord_st[$i]="";
    $this->chord_type[$i]=6;
    if ($this->mode_sig[$i] < 0) $b=1; else $b=0;
    $na=array();
    $bass = 100;
    // Create array of notes to analyse
    for ($v=0; $v<$this->vcount; $v++) {
      if ($this->vol_left[$v][$i] > 0) {
        $na[$this->note[$v][$i] % 12] += $this->mode[$i][$this->note[$v][$i] % 12]/2 + 0.5;
        // If we have not chosen bass yet, choose!
        if ($bass == 100) $bass=$this->note[$v][$i] % 12;
      }
      // Find bass note
    }
    if (count($na)==1) {
      foreach ($na as $key => $val) {
        $this->chord[$i]="=".get_notename($key, $b);
        $this->chord_type[$i]=1;
        $this->chord_i[$i]="1";
        return;
      }
    }
    if (count($na)==2) {
      $x=0;
      foreach ($na as $key => $val) {
        $na2[$x]=$key;
        $x++;
      }
      $int=abs($na2[0]-$na2[1]);
      if ($na2[0]==$bass) $mel=$na2[1];
      else $mel=$na2[0];
      $this->chord[$i]=get_notename($bass, $b).":".get_notename($mel, $b);
      $this->chord_i[$i]="2";
      $this->chord_type[$i]=2;
      return;
    }
    foreach ($this->ch_a as $key => $val) {
      for ($x=0; $x<12; $x++) {
        $cn=get_notename($x, $b).$key;
        $chv[$cn] = mul_na($val, $na, $x);
        if ($x == $bass) $chv[$cn] += 0.05;
        $ch1[$cn] = $x;
        $chk[$cn] = $key;
      }
    }
    // If did not find, make statistical choice
    if ($this->chord_type[$i] > 2) {
      arsort($chv);
      $x=0;
      foreach($chv as $key => $val) {
        if ($x==0) $dif = compare_na($this->ch_a[$chk[$key]], $na, $ch1[$key]);
        if ($dif>0) $st .= $key." (".$val.") ";
        if ($x>10) break;
        $x++;
        if ($x>1) continue;
        $this->chord[$i] = $key;
        $this->chord_i[$i]=$chk[$key];
        if ($dif == 0) {
          $this->chord_type[$i]=$this->ch_a_type[$chk[$key]];
        } else {
          $this->chord[$i] = "(".$this->chord[$i];
        }
        $this->chord_bass[$i]=($bass-$ch1[$key]) % 12;
        if ($this->chord_bass[$i]<0) $this->chord_bass[$i] += 12;
        if ($ch1[$key] != $bass) {
          $this->chord[$i] .= "/".get_notename($bass, $b);
          if ($this->chord_type[$i]<4) $this->chord_type[$i] += 1;
        }
        if ($dif>0) $this->chord[$i] .= ")";
        //break;
      }
      if ($dif>0) $this->chord_st[$i] = $st;
    }
    return $this->chord_type[$i];
  }
  
  protected function AnalyseDensity($i) {
    if ($this->mcount[$i] < 2) return 0;
    $up=0;
    $down=0;
    $same=0;
    for ($v=0; $v<$this->vcount; $v++) {
      if ($this->note[$v][$i] > $this->note[$v][$i-1]) $up += 1;
      if ($this->note[$v][$i] < $this->note[$v][$i-1]) $down += 1;
      if ($this->note[$v][$i] == $this->note[$v][$i-1]) $same += 1;
    }
    if ($up==0) $this->note_density[$i] = -1;
    if ($down==0) $this->note_density[$i]=1;
    if ($up * $down == 0) return 1;
    return 0;
  }
  
  protected function GetCharacter($i) {
    GLOBAL $t_na, $d_na, $s_na;
    $this->mel_t[$i] = 0;
    $this->mel_d[$i] = 0;
    $this->mel_s[$i] = 0;
    // Make window
    $p1=$i-$this->mel_av;
    if ($p1<0) $p1=0;
    $mode_key = $this->mode_key[$i];
    //echo "$i ";
    for ($x=$p1; $x<=$i; $x++) {
      $c = (1 - ($i-$x)/$this->mel_av);
      //echo "C:$c ";
      for ($v=0; $v<$this->vcount; $v++) {
        if ($this->hit[$x][$v]) {
          $pos = ($this->note[$v][$x] - $mode_key) % 12;
          if ($pos < 0) $pos += 12;
          //if ($t_na[$pos]>0) echo "T:".get_notename($this->note[$v][$x] % 12)." "; 
          //if ($d_na[$pos]>0) echo "D:".get_notename($this->note[$v][$x] % 12)." "; 
          //if ($s_na[$pos]>0) echo "S:".get_notename($this->note[$v][$x] % 12)." "; 
          $this->mel_t[$i] += $t_na[$pos] * $c;
          $this->mel_d[$i] += $d_na[$pos] * $c;
          $this->mel_s[$i] += $s_na[$pos] * $c;
        }
      }
    }
    //echo "<br>";
  }
  
  protected function FindHiddenTritone($i, $v) {
    $found=0;
    for ($x=1; $x<30; $x++) {
      $p1 = $i-$x;
      if ($p1<0) break;
      if ($this->note[$v][$p1] == $this->note[$v][$p1+1]) break;
      if (abs($this->note[$v][$p1] - $this->note[$v][$i]) == 6) {
        //echo "$i Masked tritone at $p1 between<br>";
        $found=$p1;
        break;
      }
    }
    return $found;
  }
  
  protected function NoteAnalyseHits($i) {
    // Count changed notes
    $rmcount=0;
    for ($v=0; $v<$this->vcount; $v++) {
      if ($this->note[$v][$i] != $this->note[$v][$i-1]) {
        $rmcount += 1;
        $this->hit[$i][$v]=1;
        $this->note_changed[$i][$v] = 1;
      } else {
        $this->hit[$i][$v]=0;
        $this->note_changed[$i][$v] = 0;
      }
    }
    $this->rmcount[$i] = $rmcount;
    // Count retriggers and hits
    $this->retrig_c[$i]=0;
    for ($v=0; $v<$this->vcount; $v++) {
      if ($this->retrigger[$v][$i]>0) {
        $this->retrig_c[$i] ++;
        $this->hit[$i][$v]=1;
      }
    }
    $this->hits[$i] = array_sum($this->hit[$i]);
  }
  
  protected function NoteAnalyse($i) {
    $this->hit[$i] = array();
    $this->NoteAnalyseHits($i);
    // Count vol_left
    for ($v=0; $v<$this->vcount; $v++) {
      $dec = 127*$this->len[$i]/$this->diss_w_past*(1 + $this->note[$v][$i]/$this->diss_w_note);
      //if ($v==0) echo "$i:$v Dec $dec Vol ".$this->vol[$v][$i]."<br>";
      //if ($v==5) echo "$i:$v __________________ Dec $dec Vol ".$this->vol[$v][$i]."<br>";
      if ($this->hit[$i][$v]) {
        $this->vol_left[$v][$i] = $this->vol[$v][$i] - $dec;
      } else {
        $this->vol_left[$v][$i] = $this->vol_left[$v][$i-1] - $dec;
      }
      if ($this->vol_left[$v][$i] < 0) $this->vol_left[$v][$i] = 0;
    }
    // Lock notes based on note_min_len and count note length
    $len=($this->len[$i]+$this->pause[$i]);
    for($v = 0; $v < $this->vcount; $v ++) {
      $rem=0;
			if (($i==0) || ($this->note[$v][$i] != $this->note[$v][$i-1]) || ($this->retrigger[$v][$i])) {
        $this->note_past[$v][$i] = 0;
			  $this->note_len[$v][$i] = $len;
			} else {
        $this->note_past[$v][$i] = $this->note_past[$v][$i-1] + $len;
			  $this->note_len[$v][$i] = $this->note_len[$v][$i-1] + $len;
			}
      $this->note_locked[$v][$i]=0;
      if (($this->note_locked[$v][$i-1]>0) && ($this->note[$v][$i] == $this->note[$v][$i-1])) {
        $rem=$this->note_locked[$v][$i-1]-$len;
        if ($rem<=0) $rem=0;
      } else {
        // Check if note moved or retriggered
        if (($this->note[$v][$i] == $this->note[$v][$i-1]) && ($this->retrigger[$v][$i] == 0)) continue;
        // Determine lock time based on note pitch
        foreach ($this->note_min_len as $key => $val) {
          //echo "$v:$i $key:$val<br>";
          if ($this->note[$v][$i] <= $key) {
            $rem=$val-$len;
            break;
          }
        }
      }
      if ($rem>0) $this->note_locked[$v][$i] = $rem;
    }
    // Analyse smothered notes
    $win1=0;
    $win2=0;
    for($v = 0; $v < $this->vcount; $v ++) {
	    // Set nchange
		  $this->nchange[$v][$i]=$this->note[$v][$i] - $this->note[$v][$i-1];
      $this->GetNoteWin($i, $v, $win1, $win2);
      $this->note_win1[$v][$i]=$win1;
      $this->note_win2[$v][$i]=$win2;
      // Count free up
      $int=0;
      $pos=$this->note[$v][$i];
      if ($win2>$pos) for ($x=1; $x<4; $x++) {
        $pos=mode_walk($this->mode[$i], $pos, 1);
        $int++;
        if ($pos>=$win2) break;
      }
      $this->note_free2[$v][$i]=$int;
      // Count free down
      $int=0;
      $pos=$this->note[$v][$i];
      if ($pos>$win1) for ($x=1; $x<4; $x++) {
        $pos=mode_walk($this->mode[$i], $pos, -1);
        $int++;
        if ($pos<=$win1) break;
      }
      $this->note_free1[$v][$i]=$int;
    }
    // Find discording notes
    $this->AnalyseDiss($i, 1);
    // Find resolution violations
    $this->FindResolutions($i, 1);
    // Find loiter
    $this->FindLoiter($i);
    // Find bad interval movements
    if ($i>0) $this->AnalyseIM($i);
    if ($i>1) $this->FindContra($i);
    // Get chords
    $this->GetChord($i);
    // Analyse orchestral density
    $this->AnalyseDensity($i);
    // Find culmination. This is done for voices probabilities. It is not complete, because does not take into account the next notes
    $this->FindCulmination($i);
    // Get melody character
    $this->GetCharacter($i);
    // Find hidden tritones
    for($v = 0; $v < $this->vcount; $v ++) {
      $this->hidden_tritone[$i][$v] = $this->FindHiddenTritone($i, $v);
    }
  }

  protected function LenMut($i) {
    $this->len[$i] = $this->len[$i-1];
    $this->len_i[$i] = $this->len_i[$i-1];
    $len=$this->len[$i];
    $len_i=$this->len_i[$i];
    $this->len_repeat[$i] = $this->len_repeat[$i-1]-1;
    //echo "$i Previous len $len len_i $len_i<br>";
    if ($this->len_repeat[$i]>0) {
      // Check if we are in len structure
      if (is_array($this->len_st_repeat[$len_i])) {
        // Get position inside structure
        $pos = count($this->len_st_repeat[$len_i]) - $this->len_repeat[$i];
        //echo "$i Len structure pos $pos<br>";
        // Change len to next len in structure
        $this->len[$i] = $this->len_st_repeat[$len_i][$pos];
      }
      return;
    }
    $len_ch_p = $this->len_ch_p[$len_i];
    foreach ($len_ch_p as $key => $val) {
      // Change probabilities at the end of the piece
      if ($i > ($this->ncount - $this->len_ch_stop - $this->len_ch_stop2)) {
        $c=(($this->len_ch_stop - ($this->ncount - $this->len_ch_stop2 - $i))/$this->len_ch_stop);
        $len_ch_p[$key] = $len_ch_p[$key]*(1-$c) + $this->len_che_p[$len_i][$key]*$c;
        //echo "$i:$key $len len_ch_p ".$len_ch_p[$key]." c $c<br>";
      }
      if ($i > ($this->ncount - $this->len_ch_stop2)) {
        $len_ch_p[$key] = $this->len_che_p[$len_i][$key];
      }
      //$this->len_che_m[$len][$key]*($this->len_ch_stop - $this->ncount + $i)/$this->len_ch_stop;
      // Prohibit choosing len if time is not lined up
      if (is_array($this->len_st_repeat[$key])) {
        $key_len = array_sum($this->len_st_repeat[$key]);
      } else {
        $key_len = $this->len_st_repeat[$key] * $key;
      }
      if (($this->time[$i] % ($key_len)) != 0) {
        //echo "$i Killed len $key having $len<br>";
        $len_ch_p[$key]=0;
      }
    }
    $len_i = rand_pa($len_ch_p);
    if ($len_i == 0) {
      print_r($len_ch_p);
      echo " <font color=red>$i Len $len_i </font><br>";
      $len_i=15; 
    }
    if (is_array($this->len_st_repeat[$len_i])) {
      $len = $this->len_st_repeat[$len_i][0];
      $this->len_repeat[$i] = count($this->len_st_repeat[$len_i]);
      //echo "$i Started len structure $len_i<br>";
    } else {
      $len = $len_i;
      $this->len_repeat[$i] = $this->len_st_repeat[$len];
    }
    // Record
    $this->len[$i]=$len;
    $this->len_i[$i]=$len_i;
  }
  
  protected function VolMut($i) {
    for ($v=0; $v<$this->vcount; $v++) {
      // Acceleration
      $avg = ($this->vol_sm_max + $this->vol_sm_min)/2;
      $range = $this->vol_sm_max - $this->vol_sm_min;
      $max=1;
      $min=-1;
      if ($this->vol_sm[$v][$i-1] > $avg) $max = 1 - ($this->vol_sm[$v][$i-1] - $avg)/$range/2;
      else $min=1 - ($avg - $this->vol_sm[$v][$i-1])/$range/2;
      $this->vol_acc[$v][$i]=$this->vol_acc[$v][$i-1]+$this->vol_acc_step*rnd($min, $max);
      if ($this->vol_acc[$v][$i]>$this->vol_acc_max) $this->vol_acc[$v][$i]=$this->vol_acc_max;
      if ($this->vol_acc[$v][$i]<$this->vol_acc_min) $this->vol_acc[$v][$i]=$this->vol_acc_min;
      // Velocity
      $this->vol_vel[$v][$i]=$this->vol_vel[$v][$i-1]+$this->vol_acc[$v][$i-1];
      if ($this->vol_vel[$v][$i]>$this->vol_vel_max) {
        //$vol_vel[$i]=0;
        $this->vol_acc[$v][$i]=-$this->vol_acc[$v][$i]/2;
        $this->vol_vel[$v][$i]=$this->vol_vel_max;
      }
      if ($this->vol_vel[$v][$i]<$this->vol_vel_min) {
        //$vol_vel[$i]=0;
        $this->vol_acc[$v][$i]=-$this->vol_acc[$v][$i]/2;
        $this->vol_vel[$v][$i]=$this->vol_vel_min;
      }
      // Volume
      $this->vol_sm[$v][$i]=$this->vol_sm[$v][$i-1]+$this->vol_vel[$v][$i-1];
      if ($this->vol_sm[$v][$i]>$this->vol_sm_max) {
        $this->vol_sm[$v][$i]=$this->vol_sm_max;
        $this->vol_vel[$v][$i]=-$this->vol_vel[$v][$i-1]/4;
        if ($this->vol_acc[$v][$i]>0) $this->vol_acc[$v][$i]=-$this->vol_acc[$v][$i-1]/4;  
      }
      if ($this->vol_sm[$v][$i]<$this->vol_sm_min) {
        $this->vol_sm[$v][$i]=$this->vol_sm_min;
        $this->vol_vel[$v][$i]=-$this->vol_vel[$v][$i-1]/4;
        if ($this->vol_acc[$v][$i]<0) $this->vol_acc[$v][$i]=-$this->vol_acc[$v][$i-1]/4;  
      }
      // Round
      $this->vol[$v][$i]= round($this->vol_sm[$v][$i]);
      // Interact with length
      $this->vol_len[$v][$i] = round((1-($this->len[$i]-$this->len_min)/($this->len_max-$this->len_min))*$this->vol_c_len);
      $this->vol[$v][$i]+= $this->vol_len[$v][$i];
      // Interact with pitch
      $this->vol_note[$v][$i] = round((($this->note[$v][$i]-$this->note_min[$i])/($this->note_max[$i]-$this->note_min[$i]))*$this->vol_c_pitch);
      $this->vol[$v][$i]+= $this->vol_note[$v][$i];
      // Interact with jump
      if ($this->note[$v][$i]*$this->note[$v][$i-1]>0) {
        $this->vol_jump[$v][$i] = round(abs($this->note[$v][$i-1]-$this->note[$v][$i])*$this->vol_c_jump);
        $this->vol[$v][$i]+= $this->vol_jump[$v][$i];
      }
      // Decrease for reprise normalized notes
      if ($this->note_normalized[$i][$v]) {
        $this->vol[$v][$i] *= $this->normalize_vol_c;
        echo "$i:$v Vol normalized to ".$this->vol[$v][$i]."<br>";
      }
      // Check absolute limits
      if ($this->vol[$v][$i]>$this->vol_max) $this->vol[$v][$i]=$this->vol_max;
      if ($this->vol[$v][$i]<$this->vol_min) $this->vol[$v][$i]=$this->vol_min;
      //echo "$v:$i a".$this->vol_acc[$v][$i]." v".$this->vol_vel[$v][$i]." ".$this->vol[$v][$i]." min".$this->vol_vel_min." max".$this->vol_vel_max."<br>";
			$this->vol2[$v][$i] = $this->vol[$v][$i] - $this->vol_sm[$v][$i];
    }
	  //echo $this->vol_sm[0][$i].";".$this->vol_vel[0][$i].";".$this->vol_acc[0][$i]."<br>"; 
  }
  
  protected function RetriggerMut($i) {
    $time1=$this->time[$i] % $this->measure;
    $this->NoteAnalyseHits($i);
    // Build probability array
    for ($v=0; $v<$this->vcount; $v++) {
      $r_p[$v] = $this->note_retrigger_p[$v];
      if (isset($this->retrigger_mul_time[$time1][$this->mcount[$i]])) $r_p[$v] *= $this->retrigger_mul_time[$time1][$this->mcount[$i]];
      if (isset($this->retrigger_mul_len[$this->len[$i]])) $r_p[$v] *= $this->retrigger_mul_len[$this->len[$i]];
      if ($this->note_same[$v][$i]>0) {
				if ($this->hits[$i] != 0) $r_p[$v] *= $this->samenote_retrigger_mul;
				  else $r_p[$v] *= $this->samenote_retrigger_mul2;
      }
      if ($this->hits[$i] != 0) if ($this->dvcount2[$v][$i]>0) {
        $r_p[$v] *= $this->retrigger_diss / $this->dvcount2[$v][$i];
        //echo "$i:$v Reduced retrigger<br>";
      }
    }
    if ($this->hits[$i] == 0) {
      // echo "$i No hits with r_p "; print_r($r_p); echo "<br>";
    }
    for ($v=0; $v<$this->vcount; $v++) {
      //echo "$v:$i ".$r_p[$v]."<br>";
      if (($this->note[$v][$i] == $this->note[$v][$i-1]) && ($this->note_locked[$v][$i-1] == 0) && (rnd(0, 99)<$r_p[$v])) {
        // Check retrigger_space_min
        $passed=1;
        if ($this->hits[$i] != 0) for ($x=$i-1; $x>= $i-$this->retrigger_space_min; $x--) {
          if ($this->hit[$x][$v] == 1) {
            //echo "$i:$v Blocked retrigger due to retrigger_space_min=".$this->retrigger_space_min." at $x<br>";
            $passed = 0;
            break;
          }
        }
        if ($passed == 0) continue;
        if ($this->dvcount2[$v][$i]>0) {
          //echo "$i:$v bad retrigger with dvcount2 ".$this->dvcount2[$v][$i].": "; print_r($r_p); echo "<br>";
        }
        $this->retrigger[$v][$i]=1;
        //echo "$i:$v Retrigger set<br>";
      }
    }
  }
  
  protected function ModeMut($i) {
    // Copy
    $this->mode[$i]=$this->mode[$i-1];
    $this->mode_key[$i]=$this->mode_key[$i-1];
    $this->mode_sig[$i]=$this->mode_sig[$i-1];
    //echo "$i-1/$i/<br>";
    // Check if end of piece
    if ($i > ($this->ncount - $this->mode_stop)) return;
    // Check if time is appropriate for mode change
    if ($this->time[$i] % $this->mode_div != 0) return;
    // Check probability
    $p = $this->mode_mut_p;
    if ($this->dia[$i-1]>0) $p *= $this->dia_mode_mul;
    if (rnd(0, 99) >= $p) return;
    // Choose how far
    $mv=mt_rand($this->mode_mut_min, $this->mode_mut_max);
    //echo "$i $mv<br>";
    // Mutate
    $this->mode_key[$i]=($this->mode_key[$i-1] + $mv*7)%12;
    if ($this->mode_key[$i]<0) $this->mode_key[$i]+=12;
    $this->mode[$i]=make_key_na($this->mode_key[$i]);
    // Count signature
    $this->mode_sig[$i] = midi_sig($this->mode_key[$i]);
    //echo "/$i/$i-1/Mut from $mode_key[$i-1] to $mode_key[$i]/<br>";
  }

	protected function TempMut($i) {
    // Acceleration
    $this->temp_acc[$i]=$this->temp_acc[$i-1]+$this->temp_acc_step*rnd(-1, 1);
    if ($this->temp_acc[$i]>$this->temp_acc_max) $this->temp_acc[$i]=$this->temp_acc_max;
    if ($this->temp_acc[$i]<$this->temp_acc_min) $this->temp_acc[$i]=$this->temp_acc_min;
    // Velocity
    $this->temp_vel[$i]=$this->temp_vel[$i-1]+$this->temp_acc[$i-1];
    if ($this->temp_vel[$i]>$this->temp_vel_max) {
      //$temp_vel[$i]=0;
      $this->temp_acc[$i]=0;
      $this->temp_vel[$i]=$this->temp_vel_max;
    }
    if ($this->temp_vel[$i]<$this->temp_vel_min) {
      //$temp_vel[$i]=0;
      $this->temp_acc[$i]=0;
      $this->temp_vel[$i]=$this->temp_vel_min;
    }
    // Tempo
    //echo "$v:$i ". $this->temp_sm[$i-1]."<br>";
    if ($i>0) {
      $this->temp_sm[$i]=$this->temp_sm[$i-1]+$this->temp_vel[$i-1];
      if ($this->temp_sm[$i]>$this->temp_sm_max) {
        $this->temp_sm[$i]=$this->temp_sm_max;
        $this->temp_vel[$i]=-$this->temp_vel[$i-1]/2;
        $this->temp_acc[$i]=-$this->temp_acc[$i-1];  
      }
      if ($this->temp_sm[$i]<$this->temp_sm_min) {
        $this->temp_sm[$i]=$this->temp_sm_min;
        $this->temp_vel[$i]=-$this->temp_vel[$i-1]/2;
        $this->temp_acc[$i]=-$this->temp_acc[$i-1];  
      }
    }
    // Round
    //$this->temp[$i]= round($this->temp_sm[$i]);
	  // Count tempo change on len
	  $win1=$i + $this->temp_len_off - $this->temp_len_r;
	  $win2=$i + $this->temp_len_off + $this->temp_len_r;
		$lena=0;
		$x2=0;
		$asum = array_sum($this->temp_lena);
		for ($x=$win1; $x<=$win2; $x++) {
		  $pos=$x;
			if ($pos<0) $pos=0;
			if ($pos>($this->ncount-1)) $pos=$this->ncount-1;
			$len = $this->len[$pos]+$this->pause[$pos];
			if (!isset($this->temp_len_c[$len])) {
			  foreach ($this->temp_len_c as $key => $val) {
			    if ($len >= $key) {
			      $mul = $val;
			      break;
			    }
			  }
			} else $mul = $this->temp_len_c[$len];
			$lena += $this->temp_lena[$x2] * $mul / $asum;
      $x2++;
		}
	  // Count tempo change on mcount
		$win1=$i + $this->temp_mcount_off - $this->temp_mcount_r;
	  $win2=$i + $this->temp_mcount_off + $this->temp_mcount_r;
	  $mcounta=0;
		$x2=0;
		$asum = array_sum($this->temp_mcounta);
		for ($x=$win1; $x<=$win2; $x++) {
		  $pos=$x;
			if ($pos<0) $pos=0;
			if ($pos>($this->ncount-1)) $pos=$this->ncount-1;
			$mcounta += $this->temp_mcounta[$x2] * $this->mcount[$pos] / $asum;
      $x2++;
		}
	  // Count tempo change on len decrease
	  $win1=$i + $this->temp_dlen_off - $this->temp_dlen_r;
	  $win2=$i + $this->temp_dlen_off + $this->temp_dlen_r;
		$dlena=0;
		$x2=0;
		$asum = array_sum($this->temp_dlena);
		$asum2 = array_sum($this->temp_dlena2);
		$asum3 = array_sum($this->temp_dlena3);
		for ($x=$win1; $x<=$win2; $x++) {
		  $pos=$x;
		  // Check if for this previous len there is special vector
		  if (isset($this->temp_dlen_c[$this->len[$pos-1]])) $temp_dlen_c = $this->temp_dlen_c[$this->len[$pos-1]];
		    else $temp_dlen_c = $this->temp_dlen_c_def; 
		  if (isset($this->temp_dlen_c2[$this->len[$pos-1]])) $temp_dlen_c2 = $this->temp_dlen_c2[$this->len[$pos-1]];
		    else $temp_dlen_c2 = $this->temp_dlen_c2_def; 
		  if (isset($this->temp_dlen_c3[$this->len[$pos-1]])) $temp_dlen_c3 = $this->temp_dlen_c3[$this->len[$pos-1]];
		    else $temp_dlen_c3 = $this->temp_dlen_c3_def; 
		  if ($pos<0) $pos=0;
			if ($pos>($this->ncount-1)) $pos=$this->ncount-1;
			$coef=$this->temp_dlena[$x2]/$asum;
			$coef2=$this->temp_dlena2[$x2]/$asum2 + $this->temp_dlena3[$x2]/$asum3;
      if ($this->len[$pos]-$this->len[$pos-1] < 0) $coef = $this->temp_dlena[$x2] * $temp_dlen_c[$this->len[$pos]]/$asum;
      if ($this->len[$pos]-$this->len[$pos-1] > 0) $coef2 = $this->temp_dlena2[$x2] * $temp_dlen_c2[$this->len[$pos]]/$asum2 + 
        $this->temp_dlena3[$x2] * $temp_dlen_c3[$this->len[$pos]]/$asum3;
			$dlena += $coef + $coef2;
      $x2++;
		}
		//echo "D $dlena<br>";
		// Count tempo change on culmination
	  $win1=$i - $this->culm_temp_r;
	  $win2=$i + $this->culm_temp_r;
		$culm=1;
		$x2=0;
		for ($x=$win1; $x<=$win2; $x++) {
		  $pos=$x;
			if ($pos<0) $pos=0;
			if ($pos>($this->ncount-1)) $pos=$this->ncount-1;
			if ($this->culm[$pos] > 0) $culm *= 1 - (1-$this->culm_temp[$x2]) / $this->culm_temp_len_mul[$this->len[$pos]];
      $x2++;
		}
		// Count tempo change on reprise
	  $win1=$i - $this->reprise_temp_r;
	  $win2=$i + $this->reprise_temp_r;
		$repr=1;
		$x2=0;
		for ($x=$win1; $x<=$win2; $x++) {
		  $pos=$x;
			if ($pos<0) $pos=0;
			if ($pos>($this->ncount-1)) $pos=$this->ncount-1;
			if ($this->reprise_first_temp[$pos] > 0) $repr *= 1 - (1-$this->reprise_temp[$x2]) / $this->reprise_temp_len_mul[$this->len[$pos]];
      $x2++;
		}
		// Record changes
	  $this->temp_len[$i]= round($lena);
	  $this->temp_mcount[$i]=$mcounta * $this->temp_mcount_c;
	  // Count target value
		$this->temp[$i]=$this->temp_a - $this->temp_len[$i] - $this->temp_mcount[$i];
		// Divide by dlen
		$this->temp_dlen[$i] = $this->temp[$i] *($dlena-3)/($dlena-2);
		$this->temp[$i] -= $this->temp_dlen[$i];
		// Add scalar sm
		$this->temp_sm2[$i] = $this->temp_sm[$i]*$this->temp_sm_c1/100;
		// Add percent sm
		$a=$this->temp_sm[$i]*$this->temp_sm_c2/10000;
		$this->temp_sm2[$i] += $this->temp[$i]*$a/(1+$a);
		$this->temp[$i] -= $this->temp_sm2[$i];
		// Multiply culmination
	  $this->temp_culm[$i]=(1-$culm) * $this->temp[$i];
	  $this->temp[$i] -= $this->temp_culm[$i];
		// Multiply reprise only if culmination did not take place here
		if ($culm == 1) {
		  $this->temp_reprise[$i]=(1-$repr) * $this->temp[$i];
		  $this->temp[$i] -= $this->temp_reprise[$i];
		}
	}
 
  protected function PauseMut($i) {
	  $time1=$this->time[$i] % $this->measure;
    $this->pause[$i]=0;
		// Check if pause is prohibited
		if (($this->pause_nodcount) && ($this->dcount[$i]>0)) return 0;
		if (($this->pause_noicount) && ($this->icount[$i]>0)) return 0;
		if (($this->pause_noccount) && ($this->ccount[$i]>0)) return 0;
		// Check for successive pauses
		for ($x=$i-1; $x>=$i - $this->pause_step_min; $x--) {
		  if ($this->pause[$x] > 0) return 0;
		}
	  $prob=$this->pause_p;
		$prob *= $this->pause_mcount_mul[$this->mcount[$i]];
		$prob *= $this->pause_len_mul[$this->len[$i]];
		if (isset($this->pause_ch_mul[$time1])) $ch_mul = $this->pause_ch_mul[$time1];
		  else $ch_mul = $this->pause_ch_mul_def;
		$prob *= $ch_mul[$this->chord_type[$i]];
		$this->pausep[$i]=$prob;
		for ($x=0; $x<10; $x++) {
		  if (rnd(0, 99) >= $prob) break;
		  $this->pause[$i] += $this->len[$i];
		  if ($this->pause[$i] >= $this->pause_max) break;
		  $prob *= $this->pause_dec/100;
		  //echo "Pause_p $i decreased to $prob<br>";
		}
	}
	
	protected function NoteAlter_Cht($i) {
	  $changed=0;
	  $passed=1;
    // Check if len is not too long
		for ($x=$i-$this->cht_len1; $x<$i+$this->cht_len2; $x++) {
	    if ($this->len[$x] > $this->cht_len_max) {
	      $passed=0;
	      break;
	    }
	  }
    // Check if tempo is not too low
		for ($x=$i-$this->cht_temp1; $x<$i+$this->cht_temp2; $x++) {
	    if ($this->temp[$x] < $this->cht_temp_min) {
      	//echo "Tempo ".$this->temp[$x]."cancelled $i cht<br>";
	      $passed=0;
	      break;
	    }
	  }
	  // Check if hit count is not too high
		for ($x=$i-$this->cht_count1; $x<$i+$this->cht_count2; $x++) {
	    if (($this->rmcount[$x]+$this->retrig_c[$x]) > $this->cht_count_max) {
      	//echo "Hit count cancelled $i cht<br>";
	      $passed=0;
	      break;
	    }
	  }
	  if ($passed) {
	    //echo "$i can be altered<br>";
	    for ($v=0; $v<$this->vcount; $v++) {
  	    $note1=$this->note[$v][$i-1];
  	    $note2=$this->note[$v][$i];
  	    $note3=$this->note[$v][$i+1];
  	    // Check if current note is not changing
  	    if ($note2 == $note1) continue;
  	    // Check if note does not change next step
  	    if ($note3 == $note2) continue;
  	    // Check if it is a turn
  	    if ((($note2-$note1)*($note3-$note2)) > 0) continue;
  	    // Find out the direction of alteration
  	    $sig = -1;
  	    if ($note2 < $note1) $sig = 1;
  	    // Check allowed directions
  	    if ($sig==1) {
  	      if (rnd(0, 99) >= $this->cht_dir_p[1]) continue;
  	    } else {
  	      if (rnd(0, 99) >= $this->cht_dir_p[0]) continue;
  	    }
  	    // Check if there is enough room for alteration
  	    if ((($v+$sig) >= 0) && (($v+$sig) < $this->vcount)) {
  	      if (abs($this->note[$v+$sig][$i] - $this->note[$v][$i]) < $this->cht_win_min) continue;
  	    }
  	    // Check if turn is too big or too small
  	    if (abs($note2-$note1) > $this->cht_jump_max) continue;
  	    if (abs($note3-$note2) > $this->cht_jump_max) continue;
  	    if (abs($note2-$note1) < $this->cht_jump_min) continue;
  	    if (abs($note3-$note2) < $this->cht_jump_min) continue;
  	    // Check probability
  	    if (rnd(0, 99) >= $this->cht_p[$v]) continue;
  	    // Check contradictions in other voices
  	    /*
  	    $passed=1;
  	    for ($x=1; $x<$this->cht_cont+1; $x++) {
      	  for ($v2=0; $v2<$this->vcount; $v2++) {
      	    // Do not check same voice, because it is allowed
      	    if ($v2==$v) continue;
      	    // Check if note equals unchanged note
  	        if ((($this->note[$v2][$i-$x] % 12) == ($note2 % 12)) && (($this->note[$v2][$i-$x-1] != $this->note[$v2][$i-$x]) || ($this->retrigger[$v2][$i-$x]))) {
      	      //echo "Cancelled $v:$i cht<br>";
  	          $passed=0;
  	          break;
  	        }
      	  }
  	    }
  	    if ($passed == 0) continue;
  	    */
  	    // Alter note
  	    $this->note[$v][$i] += $sig;
  	    $this->note_cht[$v][$i] = $sig;
  	    $this->vol[$v][$i] *= $this->cht_vol_c/100;
  	    //echo "Cht $v:$i $note2 $sig<br>";
   	    $changed += 1;
	      // Search for changed note forward
   	    $i2_max = $i+1+$this->note_alter_search;
 	      $this->alter_found[$i]=0;
   	    for ($i2=$i+1; $i2<$i2_max; $i2++) {
   	      for ($v2=0; $v2<$this->vcount; $v2++) {
   	        if ($this->hit[$v2][$i2] == 0) continue;
   	        if (($this->note[$v2][$i2] % 12) == ($this->note[$v][$i] % 12)) {
   	          echo "<font color=green><b>$i:$v Found alteration forward ".$this->note[$v][$i]." at $i2:$v2</font></b><br>";
 	            $this->alter_found[$i]=1;
   	        }
   	      }
   	    }
	    }
	  }
	  return $changed;
	}
	
	protected function NoteAlter_Chi($i) {
	  $changed=0;
		$passed=1;
    // Check if len is not too long
	  for ($x=$i-$this->chi_len1; $x<$i+$this->chi_len2; $x++) {
	    if ($this->len[$x] > $this->chi_len_max) {
	      $passed=0;
	      break;
	    }
	  }
	  // Check if tempo is not too low
		for ($x=$i-$this->chi_temp1; $x<$i+$this->chi_temp2; $x++) {
	    if ($this->temp[$x] < $this->chi_temp_min) {
	      $passed=0;
	      break;
	    }
	  }
	  // Check if hit count is not too high
		for ($x=$i-$this->chi_count1; $x<$i+$this->chi_count2; $x++) {
	    if (($this->rmcount[$x]+$this->retrig_c[$x]) > $this->chi_count_max) {
	      $passed=0;
	      break;
	    }
	  }
	  if ($passed) {
  	  for ($v=0; $v<$this->vcount; $v++) {
  	    $note1=$this->note[$v][$i-1];
  	    $note2=$this->note[$v][$i];
  	    $note3=$this->note[$v][$i+1];
  	    // Check if current note is not changing
  	    if ($note2 == $note1) continue;
  	    // Check if next note is the same
  	    if ($note3 == $note2) continue;
  	    // Check if it is not a turn
  	    if ((($note2-$note1)*($note3-$note2)) < 0) continue;
  	    // Find out the direction of alteration
  	    $sig = 1;
  	    if (($note2-$note1) < 0) $sig = -1;
  	    // Check allowed directions
  	    if ($sig==1) {
  	      if (rnd(0, 99) >= $this->chi_dir_p[1]) continue;
  	    } else {
  	      if (rnd(0, 99) >= $this->chi_dir_p[0]) continue;
  	    }
  	    // Check if there is enough room for alteration
  	    if ((($v+$sig) >= 0) && (($v+$sig) < $this->vcount)) {
  	      if (abs($this->note[$v+$sig][$i] - $this->note[$v][$i]) < $this->chi_win_min) continue;
  	    }
  	    // Check if turn is too big or too small
  	    if (abs($note3-$note2) > $this->chi_jump_max) continue;
  	    if (abs($note3-$note2) < $this->chi_jump_min) continue;
  	    // Check probability
  	    $p = $this->chi_p[$v];
  	    if ($this->dia[$i]>0) $p *= $this->chi_dia_mul/100;
  	    if (rnd(0, 99) >= $p) continue;
  	    // Check contradictions in other voices
  	    /*
  	    $passed=1;
  	    for ($x=1; $x<$this->chi_cont+1; $x++) {
      	  for ($v2=0; $v2<$this->vcount; $v2++) {
      	    // Do not check same voice, because it is allowed
      	    if ($v2==$v) continue;
      	    // Check if note equals unchanged note
  	        if ((($this->note[$v2][$i-$x] % 12) == ($note2 % 12)) && (($this->note[$v2][$i-$x-1] != $this->note[$v2][$i-$x]) || ($this->retrigger[$v2][$i-$x]))) {
  	          //echo "Cancelled $v:$i chi<br>";
  	          $passed=0;
  	          break;
  	        }
      	  }
  	    }
  	    if ($passed == 0) continue;
  	    */
  	    // Alter note
  	    $this->note[$v][$i] += $sig;
  	    $this->note_chi[$v][$i] = $sig;
  	    $this->vol[$v][$i] *= $this->chi_vol_c/100;
  	    //echo "Chi $v:$i $note2 $sig<br>";
   	    $changed += 1;
  	    // Search for changed note forward
   	    $i2_max = $i+1+$this->note_alter_search;
 	      $this->alter_found[$i]=0;
   	    for ($i2=$i+1; $i2<$i2_max; $i2++) {
   	      for ($v2=0; $v2<$this->vcount; $v2++) {
   	        if (($this->note[$v2][$i2] % 12) == ($this->note[$v][$i] % 12)) {
   	          echo "<font color=green><b>$i:$v Found alteration forward ".$this->note[$v][$i]." at $i2:$v2</font></b><br>";
 	            $this->alter_found[$i]=1;
   	        }
   	      }
   	    }
  	  }
	  }
	  return $changed;
	}
	
	protected function NoteAlter_Chs($i) {
	  $changed=0;
		$passed=1;
    // Check if len is not too long
	  for ($x=$i-$this->chs_len1; $x<$i+$this->chs_len2; $x++) {
	    if ($this->len[$x] > $this->chs_len_max) {
	      $passed=0;
	      break;
	    }
	  }
	  // Check if tempo is not too low
		for ($x=$i-$this->chs_temp1; $x<$i+$this->chs_temp2; $x++) {
	    if ($this->temp[$x] < $this->chs_temp_min) {
	      $passed=0;
	      break;
	    }
	  }
	  // Check if hit count is not too high
		for ($x=$i-$this->chs_count1; $x<$i+$this->chs_count2; $x++) {
	    if (($this->rmcount[$x]+$this->retrig_c[$x]) > $this->chs_count_max) {
	      $passed=0;
	      break;
	    }
	  }
	  if ($passed) {
  	  for ($v=0; $v<$this->vcount; $v++) {
  	    $note1=$this->note[$v][$i-1];
  	    $note2=$this->note[$v][$i];
  	    $note3=$this->note[$v][$i+1];
  	    // Check if current note is not changing
  	    if ($note2 == $note1) continue;
  	    // Check if next note is the same
  	    if ($note3 != $note2) continue;
  	    // Check if it is not a turn
  	    if ((($note2-$note1)*($note3-$note2)) < 0) continue;
  	    // Find out the direction of alteration
  	    $sig = 1;
  	    if ($note2 > $note1) $sig = -1;
  	    // Check allowed directions
  	    if ($sig==1) {
  	      if (rnd(0, 99) >= $this->chs_dir_p[1]) continue;
  	    } else {
  	      if (rnd(0, 99) >= $this->chs_dir_p[0]) continue;
  	    }
  	    // Check if there is enough room for alteration
  	    if ((($v+$sig) >= 0) && (($v+$sig) < $this->vcount)) {
  	      if (abs($this->note[$v+$sig][$i] - $this->note[$v][$i]) < $this->chs_win_min) continue;
  	    }
  	    // Check if interval is too big or too small
  	    if (abs($note2-$note1) > $this->chs_jump_max) continue;
  	    if (abs($note2-$note1) < $this->chs_jump_min) continue;
  	    // Check probability
  	    if (rnd(0, 99) >= $this->chs_p[$v]) continue;
  	    // Check contradictions in other voices
  	    /*
  	    $passed=1;
  	    for ($x=1; $x<$this->chs_cont+1; $x++) {
      	  for ($v2=0; $v2<$this->vcount; $v2++) {
      	    // Do not check same voice, because it is allowed
      	    if ($v2==$v) continue;
      	    // Check if note equals unchanged note1 and the note was hit
  	        if ((($this->note[$v2][$i-$x] % 12) == ($note1 % 12)) && (($this->note[$v2][$i-$x-1] != $this->note[$v2][$i-$x]) || ($this->retrigger[$v2][$i-$x]))) {
  	          //echo "Cancelled $v:$i chs<br>";
  	          $passed=0;
  	          break;
  	        }
      	    // Check if note equals unchanged note2 and the note was hit
  	        if ((($this->note[$v2][$i-$x] % 12) == ($note2 % 12)) && (($this->note[$v2][$i-$x-1] != $this->note[$v2][$i-$x]) || ($this->retrigger[$v2][$i-$x]))) {
  	          //echo "Cancelled $v:$i chs<br>";
  	          $passed=0;
  	          break;
  	        }
      	  }
  	    }
  	    if ($passed == 0) continue;
  	    */
  	    // Alter note
  	    $this->note[$v][$i] += $sig;
  	    $this->note_chs[$v][$i] = $sig;
  	    $this->vol[$v][$i] *= $this->chs_vol_c/100;
  	    //echo "Chs $v:$i $note2 $sig<br>";
   	    $changed += 1;
   	    // Search for changed note forward
   	    $i2_max = $i+1+$this->note_alter_search;
 	      $this->alter_found[$i]=0;
   	    for ($i2=$i+1; $i2<$i2_max; $i2++) {
   	      for ($v2=0; $v2<$this->vcount; $v2++) {
   	        if (($this->note[$v2][$i2] % 12) == ($this->note[$v][$i] % 12)) {
   	          echo "<font color=green><b>$i:$v Found alteration forward ".$this->note[$v][$i]." at $i2:$v2</font></b><br>";
   	          $this->alter_found[$i]=1;
   	        }
   	      }
   	    }
      }
	  }
	  return $changed;
	}
	
	protected function NoteAlter($i) {
	  // Save state
	  for ($v=0; $v<$this->vcount; $v++) { 
	    $note_saved[$v] = $this->note[$v][$i];
	    $vol_saved[$v] = $this->vol[$v][$i];
	  }
	  $fine_saved = $this->fine[$i];
	  // Change notes
    $changed=$this->NoteAlter_Cht($i);
    if ($changed == 0) $changed=$this->NoteAlter_Chi($i);
    if ($changed == 0) $changed=$this->NoteAlter_Chs($i);
    if ($changed == 0) return 0;
    // Check resulting fine and load state
    $fine = $this->NoteMut_Check($i, 1);
    //$lcount = $this->FindLoiter($i);
    if (($fine - $fine_saved > $this->note_alter_fine_max ) && ($this->alter_found[$i] == 0)) {
      echo "$i Reverted alteration due to fine increase from ".round($fine_saved, 2)." to ".round($fine, 2)."<br>";
  	  for ($v=0; $v<$this->vcount; $v++) { 
  	    $this->note[$v][$i] = $note_saved[$v];
  	    $this->vol[$v][$i] = $vol_saved[$v];
  	    $this->note_chi[$v][$i] = 0;
  	    $this->note_cht[$v][$i] = 0;
  	    $this->note_chs[$v][$i] = 0;
  	  }
  	  return 0;
    }
    echo "$i <b>Alteration accepted</b> due to fine change from ".round($fine_saved, 2)." to ".round($fine, 2)."<br>";
	  // Disable all retriggers
    for ($v=0; $v<$this->vcount; $v++) {
      $this->retrigger[$v][$i] = 0;
    }
	  return $changed;
	}
	
	protected function ConstMut($i) {
	  // Change note_max
	  // Check range
	  $this->note_max[$i] = $this->note_max[$i-1];
	  $nm2 = $this->note_max[$i] + $this->note_max_dir;
	  if ($nm2 > $this->note_max_max) $this->note_max_dir = -$this->note_max_dir;
	  if ($nm2 < $this->note_max_min) $this->note_max_dir = -$this->note_max_dir;
	  $nm2 = $this->note_max[$i] + $this->note_max_dir;
	  // Check closest note
	  $passed = 1;
	  if ($this->note_max_dir < 0) {
      if ($nm2 <= $this->note[$this->vcount-1][$i-1]) {
        $passed = 0;
      }
	  }
	  // If not passed then do not change and wait for the next turn
	  if ($passed) {
	    $this->note_max[$i] = $nm2;
	  }
	  // Change note_min
	  // Check range
	  $this->note_min[$i] = $this->note_min[$i-1];
	  $nm2 = $this->note_min[$i] + $this->note_min_dir;
	  if ($nm2 > $this->note_min_max) $this->note_min_dir = -$this->note_min_dir;
	  if ($nm2 < $this->note_min_min) $this->note_min_dir = -$this->note_min_dir;
	  $nm2 = $this->note_min[$i] + $this->note_min_dir;
	  // Check closest note
	  $passed = 1;
	  if ($this->note_min_dir > 0) {
      if ($nm2 >= $this->note[0][$i-1]) {
        $passed = 0;
      }
	  }
	  // If not passed then do not change and wait for the next turn
	  if ($passed) {
	    $this->note_min[$i] = $nm2;
	  }
	}

	protected function FindCulmination($i) {
	  $this->culm[$i] = 0;
	  if ($i < $this->culm_win1/2) return 0;
	  // Do not stack culmination over pause
	  if ($this->pause[$i]>0) return 0;
	  $v=$this->vcount-1;
	  // Check if enough jump
		if ($this->note[$v][$i] - $this->note[$v][$i-1] < $this->culm_jump_min) return 0;
	  $passed=1;
		$win1=$i-$this->culm_win1;
		$win2=$i+$this->culm_win2;
		if ($win1<1) $win1=1;
		if ($win2 > $this->ncount-1) $win2 = $this->ncount-1;
	  for ($x=$win1; $x < $win2; $x++) {
	    // Do not check against myself
	    if ($i == $x) continue;
	    // Check if there is higher or same height note retriggered or moved
	    if (($this->note[$v][$x] >= $this->note[$v][$i]) && (($this->note[$v][$x] != $this->note[$v][$x-1]) || ($this->retrigger[$v][$x]))) {
	      //echo "Culm $i failed by $pos<br>";
	      $passed=0;
	      break;
	    }
	  }
	  if ($passed == 0) return 0;
	  //echo "Culm $i<br>";
    //if (rnd(0, 99) >= $this->culm_p) return 0;
    $this->culm[$i] = $this->note[$v][$i];
	}

	protected function FilterCulminations() {
	  $count=count($this->culm);
	  if ($count == 0) return 0;
	  asort($this->culm);
	  $count2 = floor((100 - $this->culm_p)/100 * $count);
	  if ($count2 == 0) return 0;
	  $x=0;
	  foreach ($this->culm as $key => $val) {
	    //echo "Filtering $key ". $val."<br>";
	    $this->culm[$key]=0;
	    $x++;
	    if ($x>=$count2) break;
	  }
	  // Cancel reprise tempo changes near selected culminations
	  foreach ($this->culm as $key => $val) {
	    if ($this->culm[$key]>0) {
			  $i = $key;
	      $win1=$i - $this->reprise_temp_r;
			  $win2=$i + $this->reprise_temp_r;
				for ($x=$win1; $x<=$win2; $x++) $this->reprise_first_temp[$x] = 0;
	    }
	  }
	  //echo "Filtered: "; print_r($this->culm); echo "<br>";
	}

	protected function MakeArpeggio($i) {
	  //if ($i<1) return 0;
	  if (rnd(0, 99)>=$this->arp_p[$this->rmcount[$i]]) return 0;
	  $len = $this->len[$i] + $this->pause[$i];
		if ($len < $this->arp_len_min) return 0;
		// Count arp length
		$passed = 0;
		for ($try=0; $try<100; $try++) {
		  $incr=mt_rand($this->arp_inc_min, $this->arp_inc_max);
		  $offr=mt_rand($this->arp_off_min, $this->arp_off_max);
		  $arp_len = ($this->vcount)/2*$incr*($this->vcount) + ($this->vcount)*($this->arp_off_rnd+$offr) + ($this->arp_off_dec-1)*($this->arp_off_dec+2)/2*$offr;
		  if ($arp_len/100 > $len/2) {
		    //echo "<font color=red><b>Arp not passed: $arp_len in $len</b></font><br>";
        $this->retry_arp[$i]++;
		  } else {
		    //echo "<font color=green><b>Arp passed: $arp_len in $len</b></font><br>";
		    $passed = 1;
		    break;
		  }
		}
		if ($passed == 0) return 0;
		//echo "$i Arp len $arp_len for incr $incr offr $offr<br>";
		$this->arp[$i] = round($arp_len/100);
		$off=0;
		if ((rnd(0, 99) < $this->arp_extend_p1) && ($arp_len/100 >= $this->arp_extend_min_len1)) {
		  if ($this->note[0][$i]-12 > $this->note_min[$i]) {
		    //echo "$i Extended down <br>";
		    $this->note[-1][$i] = $this->note[0][$i]-12;
		    $this->off[-1][$i] = 0;
		    $this->vol[-1][$i] = $this->vol[0][$i] * $this->arp_extend_vol1/100;
		    // Count first off step
			  $off += $offr*($this->arp_off_dec + 1);
		  }
		}
		for ($v=0; $v<$this->vcount; $v++) {
		  //if (($this->note[$v][$i] == $this->note[$v][$i-1]) && ($this->retrigger[$v][$i] == 0)) continue;
		  // Check if we can retrigger
		  if ($this->note[$v][$i] == $this->note[$v][$i-1]) {
		    if (($this->dvcount2[$v][$i] <= $this->arp_ret_dvc_max) && (($this->note_locked[$v][$i-1] == 0) || ($this->note[$v][$i-1] > $this->arp_ret_max))) $this->retrigger[$v][$i] = 1; 
		  }
			$this->off[$v][$i] = $off;
			$offr2 = $offr;
			if ($v<$this->arp_off_dec) $offr2 *= ($this->arp_off_dec - $v);
			$off_rnd = mt_rand(-$this->arp_off_rnd, $this->arp_off_rnd);
			if (($off_rnd + $offr2) < 0) $off_rnd=0;
			$off += $offr2 + $off_rnd + $v*$incr;
			// Change volume of arp notes
			$this->vol[$v][$i] *= $this->arp_vol[$v];
			//echo "$i:$v Arp Off $off offr2 $offr2 off_rand $off_rnd incr $incr<br>";
		}
		// Extend arpeggio
		if ((rnd(0, 99) < $this->arp_extend_p2) && ($arp_len/100 >= $this->arp_extend_min_len2)) {
		  if ($this->note[$this->vcount-1][$i]+12 < $this->note_max[$i]) {
		    //echo "$i Extended up <br>";
		    $this->note[$this->vcount][$i] = $this->note[$this->vcount-1][$i]+12;
		    $this->off[$this->vcount][$i] = $off;
		    $this->vol[$this->vcount][$i] = $this->vol[$this->vcount-1][$i] * $this->arp_extend_vol2/100;
		  }
		}
		//echo "Real off is $off<br>";
		return 1;
	}
	
	function ApplyEnvelope($i) {
	  // Change starting volume
	  if ($i<$this->vol_start) for ($v=0; $v<$this->vcount; $v++) {
	    $mul = $this->vol_start_mul + $i/$this->vol_start*(1-$this->vol_start_mul);
      $this->vol[$v][$i] *= $mul;
      //echo "$v:$i Decreased vol by $mul to".This->vol[$v][$i]<br>";
	  }
	  // Change ending volume
	  if ($i>$this->ncount - $this->vol_stop) 
	    for ($v=0; $v<$this->vcount; $v++) 
	      $this->vol[$v][$i] *= $this->vol_stop_mul + (($this->ncount-$i)/$this->vol_stop*(1-$this->vol_stop_mul));
	  // Change starting tempo
	  if ($i<$this->temp_start) {
	    $mul = $this->temp_start_mul + $i/$this->temp_start*(1-$this->temp_start_mul);
      $this->temp[$i] *= $mul;
	  }
	  // Change ending tempo
	  if (($i>$this->ncount - $this->temp_stop) && ($this->temp[$this->ncount-1] > $this->temp_stop_min)) 
      $this->temp[$i] *= $this->temp_stop_mul + (($this->ncount-$i)/$this->temp_stop*(1-$this->temp_stop_mul));
	}
	
	protected function CutPiece() {
	  $found=0;
	  for ($x=$this->ncount-1; $x>=$this->ncount-$this->stop_cut; $x--) {
	    //echo "Check cut $x<br>";
	    if (($this->len[$x] >= $this->len_stop_min) || ($this->mcount[$x] >= $this->mcount_stop_min)) {
	      //echo "Found cut $x<br>";
	      $found = $x;
	      break;
	    }
	  }
	  if ($found>0) {
	    $this->ncount = $found+1;
	  }
	}
	
	protected function BringToMode($i, $i1) {
	  // Position for saving best results
    $bpos=$this->ncount+1003; // for best results
    $bpos2=$this->ncount+1004; // for first position
    $this->fine[$bpos]=100000;
    // Create matrix of movements
    $ma = array(); // [try][v]
    // Set single choice to no movement
    for ($v=0; $v<$this->vcount; $v++) $ma[0][$v] = 0;
    $found = 0;
    for ($v=0; $v<$this->vcount; $v++) {
      // Check if note moved
      if (!$this->note_changed[$i1][$v]) continue;
      // Check if note is out of mode
      $note = $this->note[$v][$i];
      $np = $note % 12; // note position in mode
      //echo "Checking note $note over mode ".$this->mode_key[$i-1]."<br>";
      if ($this->mode[$i-1][$np] == 1) continue;
      $found ++;
      // Check where we can move
	    $lim1 = 0;
	    $lim2 = 0;
	    $this->GetNoteWin($i, $v, $lim1, $lim2);
	    $p1 = 1; // possibility to move down
	    $p2 = 1; // possibility to move up
	    if ($note - $lim1 < 2) $p1 = 0;
	    if ($lim2 - $note < 2) $p2 = 0;
      // Check if we have single choice
      $c = count($ma);
	    if ($p1*$p2 == 0) {
        if ($p1 == $p2) $sign = 0;
          else if ($p1 == 0) $sign = 1;
            else $sign = -1;
        for ($try=0; $try<$c; $try++) {
          $ma[$try][$v] = $sign;
        }
      }
      // Enlarge matrix if we have multiple choices
      for ($try=0; $try<$c; $try++) {
        // Set first choice
        $ma[$try][$v] = -1;
        // Copy matrix for second choice 
        $ma[$c+$try] = $ma[$try];
        // Set second choice
        $ma[$c+$try][$v] = 1;
      }
    }
    // Check if all was in mode
    //if ($found>0) print_r($ma);
    //echo " $i BringToMode found $found choices with matrix size ".count($ma)."<br>";
    //if ($found == 0) return 0;
    // Calculate fines for each choice
    $passed = 0;
    $c = count($ma);
    $this->CopyPos($i, $bpos2);
    for($try = 0; $try < $c; $try ++) {
      // Move notes according to choice
      $this->CopyPos($bpos2, $i);
      for($v = 0; $v < $this->vcount; $v ++) {
        $this->note[$v][$i] += $ma[$try][$v];
      }
      // Count fine for choice
      $fine=$this->NoteMut_Check($i);
      //echo "$i BringToMode checking fine $fine and matrix "; print_r($ma[$try]); echo "<br>";
      $this->fine[$i]=$fine;
      if ($fine==0) {
        $passed = 1;
        break;
      } else {
        $this->retry[$i]++;
        if ($this->fine[$bpos]>$fine) {
          //echo "$i:$try:$fine:".$this->fine[$bpos]."<br>";
          $this->CopyPos($i, $bpos, 1);
          $bpos_try=$try;
        }
			  $this->sfine[$try] += $this->fine[$bpos];
      }
    }
    if ($passed==0) {
      $try = $bpos_try;
      $this->CopyPos($bpos, $i, 1);
    }
    // Detect if notes were changed
    $changed=0;
    for($v = 0; $v < $this->vcount; $v ++) {
      if ($ma[$try][$v]!=0) {
        $changed ++;
        // Retrigger changed note
        if ($this->note[$v][$i-1] == $this->note[$v][$i]) $this->retrigger[$v][$i] = 1;
      }
    }
    if ($changed>0) { 
      echo "$i BringToMode changed $changed notes with fine ".$this->fine[$i]." and matrix "; print_r($ma[$try]); echo "<br>";
    }
    return $this->fine[$i];
	}
	
	protected function StartReprise($i, $type) {
	  // Check if reprise is in action
	  if ($this->reprise_steps_left[$i]>0) return 0;
	  // Check if len structure has not finished yet
	  if ($this->len_repeat[$i-1]>1) return 0; 
	  // Check if time is appropriate for reprise start
    if ($this->time[$i] % $this->reprise_div[$type] != 0) return 0;
    // Check probability
    if (rnd(0, 99) >= $this->reprise_p[$type]) return 0;
    // Check if there is enough distance from the beginning
    if ($i <= $this->reprise_steps_min[$type]) return 0;
    if ($this->time[$i] <= $this->reprise_size_min[$type]) return 0;
    //echo "$i Reprise distance passed<br>";
    // Count maximum length of reprise without note incapsulation
    $steps_max_incap = $i;
    if (rnd(0, 99) >= $this->reprise_incap_notes[$type]) for ($x=1; $x<=$i; $x++) {
      if ($this->reprise_oldnotes[$i-$x]) {
        $steps_max_incap = min($steps_max_incap, $x-1);
        break;
      }
    }
    // Count maximum length of reprise without incapsulation
    if (rnd(0, 99) >= $this->reprise_incap[$type]) for ($x=1; $x<=$i; $x++) {
      if ($this->reprise_steps_left[$i-$x]>0) {
        $steps_max_incap = min($steps_max_incap, $x-1);
        break;
      }
    }
    // If measures are set, create measured reprise size
    if (is_array($this->reprise_measures_p[$type])) {
      $passed = 0;
      $measures_pa = $this->reprise_measures_p[$type];
      // Make probability of too big measures zero
      foreach ($this->reprise_measures_p[$type] as $key => $val) {
        if (($key * $this->measure) >= $this->time[$i]) $measures_pa[$key] = 0;
      }
      // Quit if there are no choices left
      if (array_sum($measures_pa) == 0) {
        echo "$i Reprise Measures_pa = 0 with time ".$this->time[$i]."<br>";
        return 0;
      }
      $measures = rand_pa($measures_pa);
      $size = $measures * $this->measure;
      // Count number of steps
      $steps = 0;
      for ($x=0; $x<=$i; $x++) {
        $dif = $this->time[$i] - $this->time[$i-$x] - $size;
        if ($dif == 0) {
          $steps = $x;
          break;
        }
        // Detect alignment errors
        if ($dif>0) break;
      }
      // Quit if error
      if ($steps == 0) {
        echo "$i Alignment error<br>";
        return 0;
      }
    } else {
      // Count maximum limit
      $steps_max = $this->reprise_steps_max[$type];
      // Decrease for $i
      if ($i < $steps_max) $steps_max = $i;
      $size = 0;
      for ($x=1; $x<=$steps_max; $x++) {
        // Decrease if sum of lens is too long
        if (($this->time[$i] - $this->time[$i-$x]) >= $this->reprise_size_max[$type]) {
          $steps_max = $x-1;
          break;
        }
        // Decrease if len is out of limits
        if ($this->len[$i-$x] > $this->reprise_len_max[$type]) return 0;
        if ($this->len[$i-$x] < $this->reprise_len_min[$type]) return 0;
      }
      // Quit if max < min
      if ($steps_max < $this->reprise_steps_min[$type]) return 0;
      // Create probability array and do not include notes within len groups
      $st_pa=array();
      for ($x=$this->reprise_steps_min[$type]; $x<=$steps_max; $x++) {
        if ($this->len_repeat[$i-$x-1]<2) $st_pa[$x] = 10;
      }
      // Quit if there are no choices left
      if (array_sum($st_pa) == 0) return 0;
      // Make random choice
      $steps = rand_pa($st_pa);
      //$steps = mt_rand($this->reprise_steps_min[$type], $steps_max);
    }
    // Quit if len is constant
    //echo "$i Trying reprise $steps steps, $size size, type $type, repeats $rep<br>";
    $passed = 0;
    for ($x=$i-2; $x>=$i-$steps; $x--) {
      if ($this->len[$x] != $this->len[$i-1]) {
        $passed = 1;
        break;
      }
    }
    if (!$passed) return 0;
    // Check for incapsulation
    if ($steps > $steps_max_incap) return 0;
    // Create repeat count
    $rep=mt_rand($this->reprise_rep_min[$type], $this->reprise_rep_max[$type]);
    // Save work
    $this->reprise_rep[$i] = $rep;
    $this->reprise_steps[$i] = $steps;
    $this->reprise_steps_left[$i] = $steps;
    $this->reprise_type[$i] = $type;
    $this->reprise_first[$i] = 1;
    $this->reprise_first_temp[$i] = 1;
    $this->reprise_measures[$i] = $measures;
    $this->reprise_template[$i-$steps]=$steps;
    if (rnd(0, 99) < $this->reprise_oldmode_p[$type]) $this->reprise_oldmode[$i] = 1;
    if (rnd(0, 99) < $this->reprise_oldvol_p[$type]) $this->reprise_oldvol_gen[$i] = 1;
		// Always set oldvol = 1 for first step to avoid loud note jumps
		$this->reprise_oldvol[$i] = 1;
    $this->reprise_oldnotes[$i] = 1;
    if ($this->reprise_note_finish[$type] >= 100) $this->reprise_oldnotes[$i] = 0;
    echo "$i Accepted reprise $steps steps, $size size, type $type, repeats $rep<br>";
	}
	
	protected function MakeReprise($i) {
	  // Check if we are at start
	  if ($i < $this->reprise_start) return 0;
	  // Check if we are at the end
	  if ($this->ncount - $i < $this->reprise_stop) return 0;
	  // Check if reprise was in action
	  if ($this->reprise_steps_left[$i-1]>0) {
	    // Check if we can abort reprise
      if (($this->reprise_rep[$i-1]==1) && 
          ($this->reprise_steps_left[$i-1]/$this->reprise_steps[$i-1]*100 <= $this->reprise_abort[$this->reprise_type[$i-1]]))
        return 0;
	    // Decrease steps left
	    $this->reprise_steps_left[$i] = $this->reprise_steps_left[$i-1]-1;
	    // Check if current repeat finished
	    if ($this->reprise_steps_left[$i] == 0) {
	      $this->reprise_rep[$i] = $this->reprise_rep[$i-1]-1;
	      $this->reprise_first[$i] = 1;
	      $this->reprise_first_temp[$i] = 1;
	      // Check if repeats are left
	      if ($this->reprise_rep[$i]>0) {
	        $this->reprise_steps_left[$i] = $this->reprise_steps[$i-1];
          //$this->reprise_oldnotes[$i] = 1;
          //if ($this->reprise_note_finish[$this->reprise_type[$i-1]] >= 100) $this->reprise_oldnotes[$i] = 0;
	      } else {
	        // No reprise repeats left
	      }
	    } else {
	      // If repeat is not finished
	      $this->reprise_rep[$i] = $this->reprise_rep[$i-1];
	    }
      // Abort reprise if it is last repeat
      if (($this->reprise_rep[$i] == 1) && ($this->reprise_steps_left[$i]/$this->reprise_steps[$i-1]*100 <= $this->reprise_abort[$this->reprise_type[$i]]))
        $this->reprise_steps_left[$i] = 0;
      // Check if we continue reprise
      if ($this->reprise_steps_left[$i]>0) {
        // Copy variables
        $this->reprise_template[$i - $this->reprise_steps[$i-1]] = $this->reprise_steps[$i-1];
        $this->reprise_measures[$i] = $this->reprise_measures[$i-1];
        $this->reprise_oldvol_gen[$i] = $this->reprise_oldvol_gen[$i-1];
        $this->reprise_steps[$i] = $this->reprise_steps[$i-1];
        $this->reprise_type[$i] = $this->reprise_type[$i-1];
        $this->reprise_oldmode[$i] = $this->reprise_oldmode[$i-1];
        $this->reprise_oldvol[$i] = $this->reprise_oldvol_gen[$i-1];
        //if ($this->reprise_steps_left[$i] != $this->reprise_steps[$i]) $this->reprise_oldnotes[$i] = $this->reprise_oldnotes[$i-1];
        // Allow to change notes if we are finishing reprise
        if ($this->reprise_steps_left[$i]/$this->reprise_steps[$i]*100 <= $this->reprise_note_finish[$this->reprise_type[$i]])
          $this->reprise_oldnotes[$i] = 0;
        else
          $this->reprise_oldnotes[$i] = 1;
				if (!$this->reprise_oldnotes[$i]) $this->oldvol[$i]=0;
      }
	  } else {
		  // Decide if we can start reprise
		  $this->StartReprise($i, 0);
		  $this->StartReprise($i, 1);
	  }
	  if ($this->reprise_steps_left[$i]>0) {
	    // Make reprise
	    $i1 = $i - $this->reprise_steps[$i];
	    //echo "$i Copying from $i1<br>";
	    // Copy lens
	    $this->len[$i] = $this->len[$i1];
	    $this->len_i[$i] = $this->len_i[$i1];
	    $this->len_repeat[$i] = $this->len_repeat[$i1];
	    // Copy notes and mode
	    if ($this->reprise_oldmode[$i]) {
	      $this->mode[$i] = $this->mode[$i1];
	      $this->mode_key[$i] = $this->mode_key[$i1];
	      $this->mode_sig[$i] = $this->mode_sig[$i1];
      }
      if ($this->reprise_oldnotes[$i]) {
	      $this->CopyPos($i-1, $i, 2);
	      $this->CopyPos($i1, $i, 3);
	      $this->NormalizeVoices($i);
	      // Check if copied notes are out of limits
        if ($this->note_max[$i] <= $this->note[$this->vcount-1][$i]) 
          $this->note_max[$i] = $this->note[$this->vcount-1][$i]+1;
        if ($this->note_min[$i] >= $this->note[0][$i]) 
          $this->note_min[$i] = $this->note[0][$i]-1;
        // Try to bring step to current mode
        if (!$this->reprise_oldmode[$i]) {
          $fine = $this->BringToMode($i, $i1);
	        if ($fine > $this->reprise_fine_max) {
	          // Cancel old notes if could not bring to mode
	          $this->reprise_oldnotes[$i] = 0;
	          $this->reprise_oldvol[$i] = 0;
	        }
        }
	    }
      if ($this->reprise_oldvol[$i]) {
        $this->CopyVol($i1, $i);
      }
	  }
	}
	
  function Generate() {
    $this->NoteAnalyse(0);
    // Generate music
    for($i = 1; $i < $this->ncount; $i ++) {
      $this->ConstMut($i);
      // Count time
      $this->time[$i]=$this->time[$i-1] + $this->len[$i-1];
      $this->MakeReprise($i);
      if ($this->reprise_steps_left[$i] == 0) $this->LenMut($i);
      //if (($this->reprise_steps_left[$i] == 0) || ($this->reprise_type[$i]==0)) {
      if (!$this->reprise_oldmode[$i]) $this->ModeMut($i);
      if (!$this->reprise_oldnotes[$i]) $this->NoteMut($i);
      if (!$this->reprise_oldvol[$i]) $this->VolMut($i);
      $this->RetriggerMut($i);
      $this->NoteAnalyse($i);
      // Here go functions which depend on NoteAnalyse
      $this->PauseMut($i);
      $this->RecordTimer($i);
    }
		// Change tempo on culminations
    for ($i=0; $i<$this->ncount; $i++ ) {
      $this->FindCulmination($i);
    }
    $this->FilterCulminations();
  	for ($i=0; $i<$this->ncount; $i++ ) {
      $this->TempMut($i);
		}
    // Alter notes
		for ($i=0; $i<$this->ncount; $i++ ) {
      if (($i > $this->note_alter_win1) && ($i < $this->ncount - $this->note_alter_win2)) $alter=$this->NoteAlter($i);
      if ($alter) $i += $this->note_alter_space;
		}
		// Make arpeggios
		for ($i=0; $i<$this->ncount; $i++ ) {
      $alter=$this->MakeArpeggio($i);
      if ($alter) $i += $this->arp_min_step;
		}
    // Analyse again if note was altered
		for ($i=0; $i<$this->ncount; $i++ ) {
      $this->NoteAnalyse($i);
		}
		$this->CutPiece();
		// Apply envelope
  	for ($i=0; $i<$this->ncount; $i++ ) {
		  $this->ApplyEnvelope($i);
		}
		// Count real time
		for ($i=0; $i<$this->ncount; $i++ ) {
		  $this->MakeRealTime($i);
		}
  }
}

?>
